@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -34,17 +34,13 @@ SG_USING_STD(string);
 #include "AIStatic.hxx"
 
 
-FGAIStatic::FGAIStatic(FGAIManager* mgr) {
-   manager = mgr;   
-   _type_str = "static";
-   _otype = otStatic;
+FGAIStatic::FGAIStatic() : FGAIBase(otStatic) {
 }
 
 
 FGAIStatic::~FGAIStatic() {
 }
 
-
 bool FGAIStatic::init() {
    return FGAIBase::init();
 }
