@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -320,6 +320,7 @@ void FGGround::Update(double dt) {
 				// eg " taxi holding point G2 runway "
 				trns += ConvertRwyNumToSpokenString(activeRwy);
 				if(_display) {
+					fgSetString("/sim/messages/ground", trns.c_str());
 					globals->get_ATC_display()->RegisterSingleMessage(trns, 0);
 				}
 				g->planePtr->RegisterTransmission(1);	// cleared to taxi
@@ -338,6 +339,7 @@ void FGGround::Update(double dt) {
 			sprintf(buf, "%.2f", f);
 			trns += buf;
 			if(_display) {
+				fgSetString("/sim/messages/ground", trns.c_str());
 				globals->get_ATC_display()->RegisterSingleMessage(trns, 0);
 			}
 			g->planePtr->RegisterTransmission(2);	// contact tower
