@@ -16,10 +16,13 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: replay.cxx,v 1.2 2005/11/10 10:02:00 ehofman Exp $
+// $Id: replay.cxx,v 1.5 2006-03-21 18:52:19 curt Exp $
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <simgear/constants.h>
 
@@ -275,6 +278,8 @@ static FGReplayData interpolate( double time, FGReplayData f1, FGReplayData f2 )
         result.fdm.rpm[i] = weight( fdm1.rpm[i], fdm2.rpm[i], ratio );
         result.fdm.fuel_flow[i]
             = weight( fdm1.fuel_flow[i], fdm2.fuel_flow[i], ratio );
+        result.fdm.fuel_px[i]
+            = weight( fdm1.fuel_px[i], fdm2.fuel_px[i], ratio );
         result.fdm.egt[i] = weight( fdm1.egt[i], fdm2.egt[i], ratio );
         result.fdm.cht[i] = weight( fdm1.cht[i], fdm2.cht[i], ratio );
         result.fdm.mp_osi[i] = weight( fdm1.mp_osi[i], fdm2.mp_osi[i], ratio );
