@@ -1,5 +1,9 @@
 // Calculate ILS heading
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <simgear/compiler.h>
 
 #include <stdio.h>
