@@ -17,9 +17,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: auto_gui.cxx,v 1.17 2005/11/06 14:22:02 mfranz Exp $
+// $Id: auto_gui.cxx,v 1.19 2006-02-21 01:16:06 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -644,7 +644,6 @@ void TgtAptDialog_OK (puObject *)
 int NewWaypoint( const string& Tgt_Alt )
 {
   string TgtAptId;
-  FGAirport a;
   FGFix f;
 
   double alt = 0.0;
@@ -661,15 +660,15 @@ int NewWaypoint( const string& Tgt_Alt )
   }
 
   FGRouteMgr *rm = (FGRouteMgr *)globals->get_subsystem("route-manager");
-
-  if ( fgFindAirportID( TgtAptId, &a ) ) {
+  const FGAirport *a = fgFindAirportID( TgtAptId);
+  if (a) {
 
       SG_LOG( SG_GENERAL, SG_INFO,
               "Adding waypoint (airport) = " << TgtAptId );
 
       sprintf( NewTgtAirportId, "%s", TgtAptId.c_str() );
 
-      SGWayPoint wp( a.getLongitude(), a.getLatitude(), alt,
+      SGWayPoint wp( a->getLongitude(), a->getLatitude(), alt,
                      SGWayPoint::WGS84, TgtAptId );
       rm->add_waypoint( wp );
 
