@@ -11,13 +11,16 @@
 //  WITHOUT ANY WARRANTY; without even the implied warranty of
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 //  General Public License for more details.
-// 
+//
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-//  $Id: FGMagRibbon.cxx,v 1.5 2005/10/25 13:49:57 ehofman Exp $
+//  $Id: FGMagRibbon.cxx,v 1.8 2006-02-21 01:16:09 mfranz Exp $
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include "FGMagRibbon.hxx"
 
@@ -65,8 +68,8 @@ FGMagRibbon::draw ()
 				// Adjust to put the number in the centre
   xoffset -= 0.25;
 
-  FGCroppedTexture t = getTexture();
-  t.setCrop(xoffset, yoffset, xoffset + 0.5, yoffset + 0.25);
+  FGCroppedTexture *t = getTexture();
+  t->setCrop(xoffset, yoffset, xoffset + 0.5, yoffset + 0.25);
 
   glPushAttrib(GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);
