@@ -16,9 +16,13 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: environment_ctrl.cxx,v 1.39 2005/11/14 21:59:13 mfranz Exp $
+// $Id: environment_ctrl.cxx,v 1.46 2006-03-07 10:26:25 mfranz Exp $
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <simgear/debug/logstream.hxx>
 
@@ -328,9 +332,10 @@ FGMetarEnvironmentCtrl::FGMetarEnvironmentCtrl ()
       _error_count( 0 ),
       _stale_count( 0 ),
       _dt( 0.0 ),
-      _error_dt( 0.0 )
+      _error_dt( 0.0 ),
+      last_apt(0)
 {
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
+#if defined(ENABLE_THREADS)
     thread = new MetarThread(this);
     thread->start( 1 );
 #endif // ENABLE_THREADS
@@ -338,9 +343,8 @@ FGMetarEnvironmentCtrl::FGMetarEnvironmentCtrl ()
 
 FGMetarEnvironmentCtrl::~FGMetarEnvironmentCtrl ()
 {
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
-   thread->cancel();
-   thread->join();
+#if defined(ENABLE_THREADS)
+   thread_stop();
 #endif // ENABLE_THREADS
 
    delete env;
@@ -412,7 +416,7 @@ FGMetarEnvironmentCtrl::init ()
             if ( result.m != NULL ) {
                 SG_LOG( SG_GENERAL, SG_INFO, "closest station w/ metar = "
                         << a->getId());
-                last_apt = *a;
+                last_apt = a;
                 _icao = a->getId();
                 search_elapsed = 0.0;
                 fetch_elapsed = 0.0;
@@ -470,13 +474,13 @@ FGMetarEnvironmentCtrl::update(double delta_time_sec)
                              latitude->getDoubleValue(),
                              true );
         if ( a ) {
-            if ( last_apt.getId() != a->getId()
+            if ( !last_apt || last_apt->getId() != a->getId()
                  || fetch_elapsed > same_station_interval_sec )
             {
                 SG_LOG( SG_GENERAL, SG_INFO, "closest station w/ metar = "
                         << a->getId());
                 request_queue.push( a->getId() );
-                last_apt = *a;
+                last_apt = a;
                 _icao = a->getId();
                 search_elapsed = 0.0;
                 fetch_elapsed = 0.0;
@@ -491,7 +495,7 @@ FGMetarEnvironmentCtrl::update(double delta_time_sec)
         }
     }
 
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
+#if !defined(ENABLE_THREADS)
     // No loader thread running so manually fetch the data
     string id = "";
     while ( !request_queue.empty() ) {
@@ -582,11 +586,10 @@ FGMetarEnvironmentCtrl::fetch_data( const string &icao )
     } catch (const sg_io_exception& e) {
         SG_LOG( SG_GENERAL, SG_WARN, "Error fetching live weather data: "
                 << e.getFormattedMessage().c_str() );
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
+#if defined(ENABLE_THREADS)
         if (_error_count++ >= 3) {
            SG_LOG( SG_GENERAL, SG_WARN, "Stop fetching data permanently.");
-           thread->cancel();
-           thread->join();
+           thread_stop();
         }
 #endif
 
@@ -701,37 +704,26 @@ FGMetarEnvironmentCtrl::update_metar_properties( const FGMetar *m )
 }
 
 
-#if defined(ENABLE_THREADS) && ENABLE_THREADS
-/**
- *
- */
+#if defined(ENABLE_THREADS)
+void
+FGMetarEnvironmentCtrl::thread_stop()
+{
+    request_queue.push( string() );	// ask thread to terminate
+    thread->join();
+}
+
 void
 FGMetarEnvironmentCtrl::MetarThread::run()
 {
-    pthread_cleanup_push( metar_cleanup_handler, fetcher );
     while ( true )
     {
-        set_cancel( SGThread::CANCEL_DISABLE );
-
         string icao = fetcher->request_queue.pop();
+        if (icao.empty())
+            return;
         SG_LOG( SG_GENERAL, SG_INFO, "Thread: fetch metar data = " << icao );
         FGMetarResult result = fetcher->fetch_data( icao );
-
-        set_cancel( SGThread::CANCEL_DEFERRED );
-
         fetcher->result_queue.push( result );
     }
-    pthread_cleanup_pop(1);
-}
-
-/**
- * Ensure mutex is unlocked.
- */
-void
-metar_cleanup_handler( void* arg )
-{
-    FGMetarEnvironmentCtrl* fetcher = (FGMetarEnvironmentCtrl*) arg;
-    fetcher->mutex.unlock();
 }
 #endif // ENABLE_THREADS
 
