@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: environment_mgr.cxx,v 1.18 2005/10/22 11:22:06 ehofman Exp $
+// $Id: environment_mgr.cxx,v 1.22 2006-03-23 16:48:45 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -59,6 +59,7 @@ FGEnvironmentMgr::FGEnvironmentMgr ()
 FGEnvironmentMgr::~FGEnvironmentMgr ()
 {
   delete _environment;
+  remove_subsystem("controller");
   delete _controller;
   delete fgClouds;
 }
@@ -188,6 +189,7 @@ FGEnvironmentMgr::bind ()
   fgTie("/environment/turbulence/use-cloud-turbulence", &sgEnviro,
       &SGEnviro::get_turbulence_enable_state,
       &SGEnviro::set_turbulence_enable_state);
+  sgEnviro.config(fgGetNode("/sim/rendering/precipitation"));
 }
 
 void
