@@ -40,13 +40,16 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include "FGModel.h"
-#include "FGTrim.h"
-#include "FGInitialCondition.h"
-#include "FGJSBBase.h"
-#include "FGGroundCallback.h"
-#include "FGPropertyManager.h"
-#include "FGPropagate.h"
+#include <models/FGModel.h>
+#include <models/FGOutput.h>
+#include <models/FGInput.h>
+#include <initialization/FGTrim.h>
+#include <initialization/FGInitialCondition.h>
+#include <FGJSBBase.h>
+#include <input_output/FGPropertyManager.h>
+#include <input_output/FGXMLParse.h>
+#include <input_output/FGGroundCallback.h>
+#include <models/FGPropagate.h>
 
 #include <vector>
 
@@ -54,7 +57,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_FDMEXEC "$Id: FGFDMExec.h,v 1.8 2005/05/22 08:06:47 ehofman Exp $"
+#define ID_FDMEXEC "$Id: FGFDMExec.h,v 1.11 2006-02-02 09:35:25 ehofman Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -67,17 +70,72 @@ CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 /** Encapsulates the JSBSim simulation executive.
-    This class is the interface class through which all other simulation classes
+    This class is the executive class through which all other simulation classes
     are instantiated, initialized, and run. When integrated with FlightGear (or
     other flight simulator) this class is typically instantiated by an interface
     class on the simulator side.
 
-    When an aircraft model is loaded the config file is parsed and for each of the
-    sections of the config file (propulsion, flight control, etc.) the
-    corresponding "ReadXXX()" method is called. From within this method the
-    "Load()" method of that system is called (e.g. LoadFCS).
+    At the time of simulation initialization, the interface
+    class creates an instance of this executive class. The
+    executive is subsequently directed to load the chosen aircraft specification
+    file:
 
-    <h4>JSBSim Debugging Directives</h4>
+    @code
+    fdmex = new FGFDMExec(  );
+    result = fdmex->LoadModel(  );
+    @endcode
+
+    When an aircraft model is loaded, the config file is parsed and for each of the
+    sections of the config file (propulsion, flight control, etc.) the
+    corresponding Load() method is called (e.g. FGFCS::Load()).
+
+    Subsequent to the creation of the executive and loading of the model,
+    initialization is performed. Initialization involves copying control inputs
+    into the appropriate JSBSim data storage locations, configuring it for the set
+    of user supplied initial conditions, and then copying state variables from
+    JSBSim. The state variables are used to drive the instrument displays and to
+    place the vehicle model in world space for visual rendering:
+
+    @code
+    copy_to_JSBsim(); // copy control inputs to JSBSim
+    fdmex->RunIC(); // loop JSBSim once w/o integrating
+    copy_from_JSBsim(); // update the bus
+    @endcode
+
+    Once initialization is complete, cyclic execution proceeds:
+
+    @code
+    copy_to_JSBsim(); // copy control inputs to JSBSim
+    fdmex->Run(); // execute JSBSim
+    copy_from_JSBsim(); // update the bus
+    @endcode
+
+    JSBSim can be used in a standalone mode by creating a compact stub program
+    that effectively performs the same progression of steps as outlined above for
+    the integrated version, but with two exceptions. First, the copy_to_JSBSim()
+    and copy_from_JSBSim() functions are not used because the control inputs are
+    handled directly by the scripting facilities and outputs are handled by the
+    output (data logging) class. Second, the name of a script file can be supplied
+    to the stub program. Scripting (see FGScript) provides a way to supply command
+    inputs to the simulation:
+
+    @code
+    FDMExec = new JSBSim::FGFDMExec();
+    Script = new JSBSim::FGScript(  );
+    Script->LoadScript( ScriptName ); // the script loads the aircraft and ICs
+    result = FDMExec->Run();
+    while (result) { // cyclic execution
+      if (Scripted) if (!Script->RunScript()) break; // execute script
+      result = FDMExec->Run(); // execute JSBSim
+    }
+    @endcode
+
+    The standalone mode has been useful for verifying changes before committing
+    updates to the source code repository. It is also useful for running sets of
+    tests that reveal some aspects of simulated aircraft performance, such as
+    range, time-to-climb, takeoff distance, etc.
+
+    <h3>JSBSim Debugging Directives</h3>
 
     This describes to any interested entity the debug level
     requested by setting the JSBSIM_DEBUG environment variable.
@@ -99,8 +157,15 @@ CLASS DOCUMENTATION
     - <b>16</b>: When set various parameters are sanity checked and
        a message is printed out when they go out of bounds
 
+    <h3>Properties</h3>
+    @property simulator/do_trim (write only) Can be set to the integer equivalent to one of
+                                tLongitudinal (0), tFull (1), tGround (2), tPullup (3),
+                                tCustom (4), tTurn (5). Setting this to a legal value
+                                (such as by a script) causes a trim to be performed. This
+                                property actually maps toa function call of DoTrim().
+
     @author Jon S. Berndt
-    @version $Id: FGFDMExec.h,v 1.8 2005/05/22 08:06:47 ehofman Exp $
+    @version $Revision: 1.11 $
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -122,32 +187,28 @@ public:
       FGFDMExec::Run() method must be made before the model is executed. A
       value of 1 means that the model will be executed for each call to the
       exec's Run() method. A value of 5 means that the model will only be
-      executed every 5th call to the exec's Run() method.
+      executed every 5th call to the exec's Run() method. Use of a rate other than
+      one is at this time not recommended.
       @param model A pointer to the model being scheduled.
       @param rate The rate at which to execute the model as described above.
       @return Currently returns 0 always. */
   int  Schedule(FGModel* model, int rate);
 
-  /** This executes each scheduled model in succession.
+  /** This function executes each scheduled model in succession.
       @return true if successful, false if sim should be ended  */
   bool Run(void);
 
   /** Initializes the sim from the initial condition object and executes
       each scheduled model without integrating i.e. dt=0.
-      @return true if successful
-       */
+      @return true if successful */
   bool RunIC(void);
 
-  /// Freezes the sim
-  void Freeze(void) {frozen = true;}
-
-  /// Resumes the sim
-  void Resume(void) {frozen = false;}
-
-  void SetGroundCallback(FGGroundCallback*);
+  /** Sets the ground callback pointer.
+      @param gc A pointer to a ground callback object.  */
+  void SetGroundCallback(FGGroundCallback* gc);
 
   /** Loads an aircraft model.
-      @param AircraftPath path to the aircraft directory. For instance:
+      @param AircraftPath path to the aircraft/ directory. For instance:
       "aircraft". Under aircraft, then, would be directories for various
       modeled aircraft such as C172/, x15/, etc.
       @param EnginePath path to the directory under which engine config
@@ -158,11 +219,10 @@ public:
       instance: "aircraft/x15/x15.xml"
       @param addModelToPath set to true to add the model name to the
       AircraftPath, defaults to true
-      @return true if successful*/
+      @return true if successful */
   bool LoadModel(string AircraftPath, string EnginePath, string model,
                  bool addModelToPath = true);
 
-
   /** Loads an aircraft model.  The paths to the aircraft and engine
       config file directories must be set prior to calling this.  See
       below.
@@ -175,37 +235,23 @@ public:
       @return true if successful*/
   bool LoadModel(string model, bool addModelToPath = true);
 
-
   /** Sets the path to the engine config file directories.
       @param path path to the directory under which engine config
-      files are kept, for instance "engine"
-  */
+      files are kept, for instance "engine"  */
   bool SetEnginePath(string path)   { EnginePath = path; return true; }
 
   /** Sets the path to the aircraft config file directories.
       @param path path to the aircraft directory. For instance:
       "aircraft". Under aircraft, then, would be directories for various
-      modeled aircraft such as C172/, x15/, etc.
-  */
+      modeled aircraft such as C172/, x15/, etc.  */
   bool SetAircraftPath(string path) { AircraftPath = path; return true; }
 
-  /** Sets the path to the autopilot config file directories.
-      @param path path to the control directory. For instance:
-      "control".
-  */
-//  bool SetControlPath(string path) { ControlPath = path; return true; }
-
-
   /// @name Top-level executive State and Model retrieval mechanism
   //@{
-  /// Returns the FGState pointer.
-  inline FGState* GetState(void)              {return State;}
   /// Returns the FGAtmosphere pointer.
   inline FGAtmosphere* GetAtmosphere(void)    {return Atmosphere;}
   /// Returns the FGFCS pointer.
   inline FGFCS* GetFCS(void)                  {return FCS;}
-  /// Returns the FGGroundCallback pointer.
-  inline FGGroundCallback* GetGroundCallback(void) {return GroundCallback;}
   /// Returns the FGPropulsion pointer.
   inline FGPropulsion* GetPropulsion(void)    {return Propulsion;}
   /// Returns the FGAircraft pointer.
@@ -222,41 +268,99 @@ public:
   inline FGPropagate* GetPropagate(void)      {return Propagate;}
   /// Returns the FGAuxiliary pointer.
   inline FGAuxiliary* GetAuxiliary(void)      {return Auxiliary;}
-  /// Returns the FGOutput pointer.
-  inline FGOutput* GetOutput(void)            {return Output;}
+  /// Returns the FGInput pointer.
+  inline FGInput* GetInput(void)              {return Input;}
+  /// Returns the FGGroundCallback pointer.
+  inline FGGroundCallback* GetGroundCallback(void) {return GroundCallback;}
+  /// Returns the FGState pointer.
+  inline FGState* GetState(void)              {return State;}
   // Returns a pointer to the FGInitialCondition object
   inline FGInitialCondition* GetIC(void)      {return IC;}
   // Returns a pointer to the FGTrim object
-  FGTrim* GetTrim(void);
+  inline FGTrim* GetTrim(void);
   //@}
 
   /// Retrieves the engine path.
   inline string GetEnginePath(void)          {return EnginePath;}
   /// Retrieves the aircraft path.
   inline string GetAircraftPath(void)        {return AircraftPath;}
-//  /// Retrieves the control path.
-//  inline string GetControlPath(void)        {return ControlPath;}
 
+  /// Returns the model name.
   string GetModelName(void) { return modelName; }
 
+  /// Returns a pointer to the property manager object.
   FGPropertyManager* GetPropertyManager(void);
+  /// Returns a vector of strings representing the names of all loaded models (future)
   vector <string> EnumerateFDMs(void);
+  /// Marks this instance of the Exec object as a "slave" object.
   void SetSlave(void) {IsSlave = true;}
 
+  /** Executes trimming in the selected mode.
+  *   @param mode Specifies how to trim:
+  * - tLongitudinal=0
+  * - tFull
+  * - tGround
+  * - tPullup
+  * - tCustom
+  * - tTurn
+  * - tNone  */
+  void DoTrim(int mode);
+
+  /// Disables data logging to all outputs.
+  void DisableOutput(void);
+  /// Enables data logging to all outputs.
+  void EnableOutput(void);
+  /// Pauses execution by preventing time from incrementing.
+  void Hold(void) {holding = true;}
+  /// Resumes execution from a "Hold".
+  void Resume(void) {holding = false;}
+  /// Returns true if the simulation is Holding (i.e. simulation time is not moving).
+  bool Holding(void) {return holding;}
+
+  struct PropertyCatalogStructure {
+    /// Name of the property.
+    string base_string;
+    /// The node for the property.
+    FGPropertyManager *node;
+  };
+
+  /** Builds a catalog of properties.
+  *   This function descends the property tree and creates a list (an STL vector)
+  *   containing the name and node for all properties.
+  *   @param pcs The "root" property catalog structure pointer.  */
+  void BuildPropertyCatalog(struct PropertyCatalogStructure* pcs);
+
+  /** Retrieves property or properties matching the supplied string.
+  *   A string is returned that contains a carriage return delimited list of all
+  *   strings in the property catalog that matches the supplied check string.
+  *   @param check The string to search for in the property catalog.
+  *   @return the carriage-return-delimited string containing all matching strings
+  *               in the catalog.  */
+  string QueryPropertyCatalog(string check);
+
+  /// Use the MSIS atmosphere model.
+  void UseAtmosphereMSIS(void);
+
+  /// Use the Mars atmosphere model. (Not operative yet.)
+  void UseAtmosphereMars(void);
+
 private:
   FGModel* FirstModel;
 
-  bool frozen;
   bool terminate;
+  bool holding;
+  bool Constructing;
   int  Error;
   unsigned int Frame;
   unsigned int IdFDM;
+  FGPropertyManager* Root;
   static unsigned int FDMctr;
   bool modelLoaded;
   string modelName;
   bool IsSlave;
   static FGPropertyManager *master;
   FGPropertyManager *instance;
+  vector <string> PropertyCatalog;
 
   struct slaveData {
     FGFDMExec* exec;
@@ -279,15 +383,14 @@ private:
 
   string AircraftPath;
   string EnginePath;
-//  string ControlPath;
 
   string CFGVersion;
   string Release;
 
+  FGGroundCallback*  GroundCallback;
   FGState*           State;
   FGAtmosphere*      Atmosphere;
   FGFCS*             FCS;
-  FGGroundCallback*  GroundCallback;
   FGPropulsion*      Propulsion;
   FGMassBalance*     MassBalance;
   FGAerodynamics*    Aerodynamics;
@@ -296,21 +399,17 @@ private:
   FGAircraft*        Aircraft;
   FGPropagate*       Propagate;
   FGAuxiliary*       Auxiliary;
-  FGOutput*          Output;
+  FGInput*           Input;
+  vector <FGOutput*> Outputs;
 
   FGInitialCondition* IC;
   FGTrim *Trim;
 
   vector <slaveData*> SlaveFDMList;
 
-  bool ReadMetrics(FGConfigFile*);
-  bool ReadSlave(FGConfigFile*);
-  bool ReadPropulsion(FGConfigFile*);
-  bool ReadFlightControls(FGConfigFile*);
-  bool ReadAerodynamics(FGConfigFile*);
-  bool ReadUndercarriage(FGConfigFile*);
-  bool ReadPrologue(FGConfigFile*);
-  bool ReadOutput(FGConfigFile*);
+  bool ReadFileHeader(Element*);
+  bool ReadSlave(Element*);
+  bool ReadPrologue(Element*);
 
   bool Allocate(void);
   bool DeAllocate(void);
