@@ -59,26 +59,25 @@ INCLUDES
 #include <string>
 #include <map>
 #include "FGJSBBase.h"
-#include "FGInitialCondition.h"
-#include "FGMatrix33.h"
-#include "FGColumnVector3.h"
-#include "FGQuaternion.h"
+#include <initialization/FGInitialCondition.h>
+#include <math/FGMatrix33.h>
+#include <math/FGColumnVector3.h>
+#include <math/FGQuaternion.h>
 #include "FGFDMExec.h"
-#include "FGAtmosphere.h"
-#include "FGFCS.h"
-#include "FGPropagate.h"
-#include "FGAuxiliary.h"
-#include "FGAerodynamics.h"
-#include "FGOutput.h"
-#include "FGAircraft.h"
-#include "FGGroundReactions.h"
-#include "FGPropulsion.h"
+#include <models/FGAtmosphere.h>
+#include <models/FGFCS.h>
+#include <models/FGPropagate.h>
+#include <models/FGAuxiliary.h>
+#include <models/FGAerodynamics.h>
+#include <models/FGAircraft.h>
+#include <models/FGGroundReactions.h>
+#include <models/FGPropulsion.h>
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_STATE "$Id: FGState.h,v 1.9 2004/06/14 11:40:45 ehofman Exp $"
+#define ID_STATE "$Id: FGState.h,v 1.11 2006-02-02 09:35:25 ehofman Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -91,8 +90,10 @@ CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 /** Encapsulates the calculation of aircraft state.
+    <h3>Properties</h3>
+    @property sim-time-sec (read only) cumulative simulation in seconds.
     @author Jon S. Berndt
-    @version $Id: FGState.h,v 1.9 2004/06/14 11:40:45 ehofman Exp $
+    @version $Revision: 1.11 $
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -110,37 +111,39 @@ public:
 
   /** Initializes the simulation state based on parameters from an Initial Conditions object.
       @param FGIC pointer to an initial conditions object.
-      @see FGInitialConditions.
-      */
+      @see FGInitialConditions.    */
   void Initialize(FGInitialCondition *FGIC);
 
-  /// Returns the simulation time in seconds.
+  /// Returns the cumulative simulation time in seconds.
   inline double Getsim_time(void) const { return sim_time; }
+
   /// Returns the simulation delta T.
-  inline double Getdt(void) { return dt; }
+  inline double Getdt(void) {return dt;}
 
   /// Suspends the simulation and sets the delta T to zero.
-  inline void Suspend(void) {saved_dt = dt; dt = 0.0;}
+  inline void SuspendIntegration(void) {saved_dt = dt; dt = 0.0;}
+
   /// Resumes the simulation by resetting delta T to the correct value.
-  inline void Resume(void)  {dt = saved_dt;}
+  inline void ResumeIntegration(void)  {dt = saved_dt;}
+
+  /** Returns the simulation suspension state.
+      @return true if suspended, false if executing  */
+  bool IntegrationSuspended(void) {return dt == 0.0;}
 
   /** Sets the current sim time.
       @param cur_time the current time
-      @return the current time.
-      */
+      @return the current simulation time.      */
   inline double Setsim_time(double cur_time) {
     sim_time = cur_time;
     return sim_time;
   }
 
   /** Sets the integration time step for the simulation executive.
-      @param delta_t the time step in seconds.
-      */
+      @param delta_t the time step in seconds.     */
   inline void  Setdt(double delta_t) { dt = delta_t; }
 
   /** Increments the simulation time.
-      @return the new simulation time.
-      */
+      @return the new simulation time.     */
   inline double IncrTime(void) {
     sim_time+=dt;
     return sim_time;
@@ -157,13 +160,9 @@ public:
   FGMatrix33& GetTb2s(void);
 
   /** Prints a summary of simulator state (speed, altitude,
-      configuration, etc.)
-  */
+      configuration, etc.)  */
   void ReportState(void);
 
-  void bind();
-  void unbind();
-
 private:
   double sim_time, dt;
   double saved_dt;
@@ -174,7 +173,6 @@ private:
 
   FGAircraft* Aircraft;
   FGPropagate* Propagate;
-  FGOutput* Output;
   FGAtmosphere* Atmosphere;
   FGFCS* FCS;
   FGAerodynamics* Aerodynamics;
@@ -183,6 +181,9 @@ private:
   FGAuxiliary* Auxiliary;
   FGPropertyManager* PropertyManager;
 
+  void bind();
+  void unbind();
+
   void Debug(int from);
 };
 }
