@@ -16,9 +16,13 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: LaRCsim.cxx,v 1.13 2005/11/01 13:41:50 ehofman Exp $
+// $Id: LaRCsim.cxx,v 1.15 2006-02-21 17:45:03 mfranz Exp $
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <math.h>
 #include <string.h>		// strcmp()
