@@ -86,8 +86,11 @@
 
 	CURRENT RCS HEADER:
 
-$Header: /h/curt/CVS/FlightGear-0.7/FlightGear/src/FDM/LaRCsim/ls_trim.c,v 1.2 2001/07/30 20:53:54 curt Exp $
+$Header: /var/cvs/FlightGear-0.9/FlightGear/src/FDM/LaRCsim/ls_trim.c,v 1.1.1.1 2002/09/10 01:14:02 curt Exp $
 $Log: ls_trim.c,v $
+Revision 1.1.1.1  2002/09/10 01:14:02  curt
+Initial revision of FlightGear-0.9.0
+
 Revision 1.2  2001/07/30 20:53:54  curt
 Various MSVC tweaks and warning fixes.
 
@@ -159,7 +162,7 @@ Start of 0.6.x branch.
 
 --------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: ls_trim.c,v 1.2 2001/07/30 20:53:54 curt Exp $";
+static char rcsid[] = "$Id: ls_trim.c,v 1.1.1.1 2002/09/10 01:14:02 curt Exp $";
 
 #ifdef __SUNPRO_CC
 #  define _REENTRANT
