@@ -1,5 +1,5 @@
 /**********************************************************************
-                                                                       
+
  FILENAME:     uiuc_menu_Croll.cpp
 
 ----------------------------------------------------------------------
@@ -66,8 +66,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- USA or view http://www.gnu.org/copyleft/gpl.html.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 **********************************************************************/
 
