@@ -10,14 +10,12 @@
 SG_USING_STD(list);
 SG_USING_STD(string);
 SG_USING_STD(getline);
-#if !defined (SG_HAVE_NATIVE_SGI_COMPILERS)
 SG_USING_STD(ifstream);
-#endif
 
 #define DELIMITERS " \t"
 #define COMMENT "#"
 
-#define MAXLINE 200   // Max size of the line of the input file
+#define MAXLINE 400   // Max size of the line of the input file
 
 typedef list<string> stack; //list to contain the input file "command_lines"
 
