@@ -10,6 +10,7 @@
 class Vector {
 public:
     Vector();
+    ~Vector();
     int   add(void* p);
     void* get(int i);
     void  set(int i, void* p);
@@ -29,6 +30,11 @@ inline Vector::Vector()
     _array = 0;
 }
 
+inline Vector::~Vector()
+{
+    delete[] _array;
+}
+
 inline int Vector::add(void* p)
 {
     if(_nelem == _sz)
