@@ -20,13 +20,13 @@
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      Library General Public License for more details.
  
-     You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
-     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- 
+     You should have received a copy of the GNU General Public License
+     along with this program; if not, write to the Free Software
+     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+
      For further information visit http://plib.sourceforge.net
 
-     $Id: prop_picker.cxx,v 1.9 2005/11/06 14:24:03 mfranz Exp $
+     $Id: prop_picker.cxx,v 1.10 2006-02-21 01:18:36 mfranz Exp $
      ********
 */
 
