@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: input.cxx,v 1.70 2005/11/12 10:51:58 mfranz Exp $
+// $Id: input.cxx,v 1.85 2006-03-21 17:33:54 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -57,6 +57,11 @@
 
 #include "input.hxx"
 
+#include <Scenery/scenery.hxx>
+#include <Main/renderer.hxx>
+#include <plib/ssg.h>
+#include <simgear/math/sg_geodesy.hxx>
+
 SG_USING_STD(ifstream);
 SG_USING_STD(string);
 SG_USING_STD(vector);
@@ -273,7 +278,7 @@ FGInput::doKey (int k, int modifiers, int x, int y)
         if (_key_bindings[k - '@'].last_state)
           doKey(k - '@', KEYMOD_RELEASED, x, y);
         if (_key_bindings[tolower(k)].last_state)
-           doKey(tolower(k), KEYMOD_RELEASED, x, y);
+          doKey(tolower(k), KEYMOD_RELEASED, x, y);
       } else if (k >= 'a' && k <= 'z') {
         if (_key_bindings[k - '`'].last_state)
           doKey(k - '`', KEYMOD_RELEASED, x, y);
@@ -308,6 +313,26 @@ FGInput::doMouseClick (int b, int updown, int x, int y)
       return;
     else if (fgHandle3DPanelMouseEvent(b, updown, x, y))
       return;
+    else {
+      // pui and the panel didn't want the click event so compute a
+      // terrain intersection point corresponding to the mouse click
+      // and be happy.
+      FGScenery* scenery = globals->get_scenery();
+      sgdVec3 start, dir, hit;
+      if (!b && updown == MOUSE_BUTTON_DOWN && FGRenderer::getPickInfo(start, dir, x, y)
+          && scenery->get_cart_ground_intersection(start, dir, hit)) {
+
+        Point3D geod = sgCartToGeod(Point3D(hit[0], hit[1], hit[2]));
+
+        SGPropertyNode *c = fgGetNode("/sim/input/click", true);
+        c->setDoubleValue("longitude-deg", geod.lon() * SGD_RADIANS_TO_DEGREES);
+        c->setDoubleValue("latitude-deg", geod.lat() * SGD_RADIANS_TO_DEGREES);
+        c->setDoubleValue("elevation-m", geod.elev());
+        c->setDoubleValue("elevation-ft", geod.elev() * SG_METER_TO_FEET);
+
+        fgSetBool("/sim/signals/click", 1);
+      }
+    }
   }
 
                                 // OK, PUI and the panel didn't want the click
@@ -364,19 +389,13 @@ FGInput::doMouseMotion (int x, int y)
                                 // Constrain the mouse if requested
   if (mode.constrained) {
     bool need_warp = false;
-    if (x <= 0) {
-      x = xsize - 2;
-      need_warp = true;
-    } else if (x >= (xsize-1)) {
-      x = 1;
+    if (x <= (xsize * .25) || x >= (xsize * .75)) {
+      x = int(xsize * .5);
       need_warp = true;
     }
 
-    if (y <= 0) {
-      y = ysize - 2;
-      need_warp = true;
-    } else if (y >= (ysize-1)) {
-      y = 1;
+    if (y <= (ysize * .25) || y >= (ysize * .75)) {
+      y = int(ysize * .5);
       need_warp = true;
     }
 
@@ -973,6 +992,8 @@ FGInput::_find_key_bindings (unsigned int k, int modifiers)
 
 FGInput::button::button ()
   : is_repeatable(false),
+    interval_sec(0),
+    last_dt(0),
     last_state(0)
 {
 }
@@ -995,7 +1016,9 @@ FGInput::axis::axis ()
   : last_value(9999999),
     tolerance(0.002),
     low_threshold(-0.9),
-    high_threshold(0.9)
+    high_threshold(0.9),
+    interval_sec(0),
+    last_dt(0)
 {
 }
 
@@ -1013,6 +1036,12 @@ FGInput::axis::~axis ()
 ////////////////////////////////////////////////////////////////////////
 
 FGInput::joystick::joystick ()
+  : jsnum(0),
+    js(0),
+    naxes(0),
+    nbuttons(0),
+    axes(0),
+    buttons(0)
 {
 }
 
