@@ -3,6 +3,10 @@
 //
 // This file is in the Public Domain and comes with no warranty.
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <iostream>
 #include <string>
 #include <sstream>
@@ -39,6 +43,7 @@
 #include "od_gauge.hxx"
 #include "wxradar.hxx"
 #include "tacan.hxx" 
+#include "mk_viii.hxx"
 
 
 FGInstrumentMgr::FGInstrumentMgr ()
@@ -161,6 +166,9 @@ bool FGInstrumentMgr::build ()
         } else if ( name == "tacan" ) { 
             set_subsystem( "instrument" + temp.str(), 
                            new TACAN( node ) );
+        } else if ( name == "mk-viii" ) { 
+            set_subsystem( "instrument" + temp.str(), 
+                           new MK_VIII( node ) );
         } else {
             SG_LOG( SG_ALL, SG_ALERT, "Unknown top level section: " 
                     << name );
