@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: marker_beacon.cxx,v 1.4 2005/05/12 18:30:59 mfranz Exp $
+// $Id: marker_beacon.cxx,v 1.7 2006-03-18 07:16:18 frohlich Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -114,6 +114,8 @@ FGMarkerBeacon::init ()
     morse.init();
     beacon.init();
     blink.stamp();
+
+    outer_marker = middle_marker = inner_marker = false;
 }
 
 
@@ -293,8 +295,10 @@ void FGMarkerBeacon::search()
         if ( last_beacon != OUTER ) {
             if ( ! globals->get_soundmgr()->exists( "outer-marker" ) ) {
                 SGSoundSample *sound = beacon.get_outer();
-                sound->set_volume( 0.3 );
-                globals->get_soundmgr()->add( sound, "outer-marker" );
+                if ( sound ) {
+                    sound->set_volume( 0.3 );
+                    globals->get_soundmgr()->add( sound, "outer-marker" );
+                }
             }
         }
         if ( audio_btn->getBoolValue() ) {
@@ -310,8 +314,10 @@ void FGMarkerBeacon::search()
 	if ( last_beacon != MIDDLE ) {
 	    if ( ! globals->get_soundmgr()->exists( "middle-marker" ) ) {
 		SGSoundSample *sound = beacon.get_middle();
-		sound->set_volume( 0.3 );
-		globals->get_soundmgr()->add( sound, "middle-marker" );
+                if ( sound ) {
+		    sound->set_volume( 0.3 );
+		    globals->get_soundmgr()->add( sound, "middle-marker" );
+                }
 	    }
         }
         if ( audio_btn->getBoolValue() ) {
@@ -327,8 +333,10 @@ void FGMarkerBeacon::search()
 	if ( last_beacon != INNER ) {
 	    if ( ! globals->get_soundmgr()->exists( "inner-marker" ) ) {
 		SGSoundSample *sound = beacon.get_inner();
-		sound->set_volume( 0.3 );
-		globals->get_soundmgr()->add( sound, "inner-marker" );
+                if ( sound ) {
+		    sound->set_volume( 0.3 );
+		    globals->get_soundmgr()->add( sound, "inner-marker" );
+                }
 	    }
         }
         if ( audio_btn->getBoolValue() ) {
