@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: bootstrap.cxx,v 1.16 2005/06/12 16:01:49 mfranz Exp $
+// $Id: bootstrap.cxx,v 1.22 2006-03-28 21:18:32 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -28,8 +28,11 @@
 #if defined(__linux__) && defined(__i386__)
 #  include <fpu_control.h>
 #  include <signal.h>
+#elif defined(__FreeBSD__)
+#  include <signal.h>
 #endif
 
+#include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -56,6 +59,10 @@ SG_USING_STD(endl);
 #  include <console.h>		// -dw- for command line dialog
 #endif
 
+char *homedir = ::getenv( "HOME" );
+char *hostname = ::getenv( "HOSTNAME" );
+bool free_hostname = false;
+
 // foreward declaration.
 void fgExitCleanup();
 
@@ -142,9 +149,12 @@ int main ( int argc, char **argv ) {
 
     _bootstrap_OSInit = 0;
 
-    // Enable floating-point exceptions for Linux/x86
 #if defined(__linux__) && defined(__i386__)
+    // Enable floating-point exceptions for Linux/x86
     initFPE();
+#elif defined(__FreeBSD__)
+    // Ignore floating-point exceptions on FreeBSD
+    signal(SIGFPE, SIG_IGN);
 #endif
 
 #if defined(sgi)
@@ -191,7 +201,7 @@ int main ( int argc, char **argv ) {
     try {
         atexit(fgExitCleanup);
         fgMainInit(argc, argv);
-    } catch (sg_throwable &t) {
+    } catch (const sg_throwable &t) {
                             // We must use cerr rather than
                             // logging, since logging may be
                             // disabled.
@@ -199,9 +209,13 @@ int main ( int argc, char **argv ) {
         if (!t.getOrigin().empty())
             cerr << " (received from " << t.getOrigin() << ')' << endl;
 
+    } catch (const string &s) {
+        cerr << "Fatal error: " << s << endl;
+
     } catch (...) {
         cerr << "Unknown exception in the main loop. Aborting..." << endl;
-        perror("Possible cause");
+        if (errno)
+            perror("Possible cause");
     }
 
     return 0;
@@ -215,5 +229,8 @@ void fgExitCleanup() {
         fgSetMouseCursor(MOUSE_CURSOR_POINTER);
 
     delete globals;
+
+    if (free_hostname && hostname != NULL)
+        free(hostname);
 }
 
