@@ -16,10 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-//
-// $Id: fg_init.cxx,v 1.138 2005/11/11 15:08:18 mfranz Exp $
+// $Id: fg_init.cxx,v 1.161 2006-04-02 08:42:36 fredb Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +34,6 @@
 #include <stdlib.h>
 #include <string.h>             // strcmp()
 
-
 #if defined( unix ) || defined( __CYGWIN__ )
 #  include <unistd.h>           // for gethostname()
 #endif
@@ -106,15 +104,13 @@
 #include <Sound/fg_fx.hxx>
 #include <Sound/beacon.hxx>
 #include <Sound/morse.hxx>
+#include <Sound/voice.hxx>
 #include <Systems/system_mgr.hxx>
 #include <Time/light.hxx>
 #include <Time/sunsolver.hxx>
 #include <Time/tmp.hxx>
 #include <Traffic/TrafficMgr.hxx>
-
-#ifdef FG_MPLAYER_AS
 #include <MultiPlayer/multiplaymgr.hxx>
-#endif
 
 #include <Environment/environment_mgr.hxx>
 
@@ -126,6 +122,7 @@
 #include "globals.hxx"
 #include "logger.hxx"
 #include "viewmgr.hxx"
+#include "main.hxx"
 
 #if defined(FX) && defined(XMESA)
 #include <GL/xmesa.h>
@@ -144,6 +141,14 @@ float init_volume;
 static string fgScanForOption( const string& option, int argc, char **argv ) {
     int i = 1;
 
+    if (hostname == NULL)
+    {
+        char _hostname[256];
+        gethostname(_hostname, 256);
+        hostname = strdup(_hostname);
+        free_hostname = true;
+    }
+
     SG_LOG(SG_GENERAL, SG_INFO, "Scanning command line for: " << option );
 
     int len = option.length();
@@ -206,46 +211,60 @@ static string fgScanForOption( const string& option, const string& path ) {
     return "";
 }
 
-
-// Read in configuration (files and command line options) but only set
-// fg_root
-bool fgInitFGRoot ( int argc, char **argv ) {
-    string root;
-    char* envp;
-
-    // First parse command line options looking for --fg-root=, this
-    // will override anything specified in a config file
-    root = fgScanForOption( "--fg-root=", argc, argv);
+// Scan the user config files for the specified option and return
+// the value.
+static string fgScanForOption( const string& option ) {
+    string arg("");
 
 #if defined( unix ) || defined( __CYGWIN__ )
     // Next check home directory for .fgfsrc.hostname file
-    if ( root.empty() ) {
-        envp = ::getenv( "HOME" );
-        if ( envp != NULL ) {
-            SGPath config( envp );
+    if ( arg.empty() ) {
+        if ( homedir != NULL ) {
+            SGPath config( homedir );
             config.append( ".fgfsrc" );
-            char name[256];
-            gethostname( name, 256 );
             config.concat( "." );
-            config.concat( name );
-            root = fgScanForOption( "--fg-root=", config.str() );
+            config.concat( hostname );
+            arg = fgScanForOption( option, config.str() );
         }
     }
 #endif
 
     // Next check home directory for .fgfsrc file
-    if ( root.empty() ) {
-        envp = ::getenv( "HOME" );
-        if ( envp != NULL ) {
-            SGPath config( envp );
+    if ( arg.empty() ) {
+        if ( homedir != NULL ) {
+            SGPath config( homedir );
             config.append( ".fgfsrc" );
-            root = fgScanForOption( "--fg-root=", config.str() );
+            arg = fgScanForOption( option, config.str() );
         }
     }
+
+    if ( arg.empty() ) {
+        // Check for $fg_root/system.fgfsrc
+        SGPath config( globals->get_fg_root() );
+        config.append( "system.fgfsrc" );
+        arg = fgScanForOption( option, config.str() );
+    }
+
+    return arg;
+}
+
+
+// Read in configuration (files and command line options) but only set
+// fg_root
+bool fgInitFGRoot ( int argc, char **argv ) {
+    string root;
+
+    // First parse command line options looking for --fg-root=, this
+    // will override anything specified in a config file
+    root = fgScanForOption( "--fg-root=", argc, argv);
+
+    // Check in one of the user configuration files.
+    if (root.empty() )
+        root = fgScanForOption( "--fg-root=" );
     
     // Next check if fg-root is set as an env variable
     if ( root.empty() ) {
-        envp = ::getenv( "FG_ROOT" );
+        char *envp = ::getenv( "FG_ROOT" );
         if ( envp != NULL ) {
             root = envp;
         }
@@ -295,44 +314,15 @@ bool fgInitFGRoot ( int argc, char **argv ) {
 // aircraft
 bool fgInitFGAircraft ( int argc, char **argv ) {
     string aircraft;
-    char* envp;
 
     // First parse command line options looking for --aircraft=, this
     // will override anything specified in a config file
     aircraft = fgScanForOption( "--aircraft=", argc, argv );
 
-#if defined( unix ) || defined( __CYGWIN__ )
-    // Next check home directory for .fgfsrc.hostname file
-    if ( aircraft.empty() ) {
-        envp = ::getenv( "HOME" );
-        if ( envp != NULL ) {
-            SGPath config( envp );
-            config.append( ".fgfsrc" );
-            char name[256];
-            gethostname( name, 256 );
-            config.concat( "." );
-            config.concat( name );
-            aircraft = fgScanForOption( "--aircraft=", config.str() );
-        }
-    }
-#endif
+    // Check in one of the user configuration files.
+    if ( aircraft.empty() )
+        aircraft = fgScanForOption( "--aircraft=" );
 
-    // Next check home directory for .fgfsrc file
-    if ( aircraft.empty() ) {
-        envp = ::getenv( "HOME" );
-        if ( envp != NULL ) {
-            SGPath config( envp );
-            config.append( ".fgfsrc" );
-            aircraft = fgScanForOption( "--aircraft=", config.str() );
-        }
-    }
-
-    if ( aircraft.empty() ) {
-        // Check for $fg_root/system.fgfsrc
-        SGPath sysconf( globals->get_fg_root() );
-        sysconf.append( "system.fgfsrc" );
-        aircraft = fgScanForOption( "--aircraft=", sysconf.str() );
-    }
     // if an aircraft was specified, set the property name
     if ( !aircraft.empty() ) {
         SG_LOG(SG_INPUT, SG_INFO, "aircraft = " << aircraft );
@@ -493,27 +483,22 @@ do_options (int argc, char ** argv)
     fgParseOptions(config.str());
 
 #if defined( unix ) || defined( __CYGWIN__ )
-    char name[256];
-    // Check for $fg_root/system.fgfsrc.hostname
-    gethostname( name, 256 );
     config.concat( "." );
-    config.concat( name );
+    config.concat( hostname );
     fgParseOptions(config.str());
 #endif
 
     // Check for ~/.fgfsrc
-    char* envp = ::getenv( "HOME" );
-    if ( envp != NULL ) {
-        config.set( envp );
+    if ( homedir != NULL ) {
+        config.set( homedir );
         config.append( ".fgfsrc" );
         fgParseOptions(config.str());
     }
 
 #if defined( unix ) || defined( __CYGWIN__ )
     // Check for ~/.fgfsrc.hostname
-    gethostname( name, 256 );
     config.concat( "." );
-    config.concat( name );
+    config.concat( hostname );
     fgParseOptions(config.str());
 #endif
 
@@ -531,27 +516,28 @@ static string fgFindAircraftPath( const SGPath &path, const string &aircraft ) {
         exit(-1);
     }
 
+    string result;
     while ((dire = ulReadDir(dirp)) != NULL) {
         if (dire->d_isdir) {
             if ( strcmp("CVS", dire->d_name) && strcmp(".", dire->d_name)
-                 && strcmp("..", dire->d_name) )
+                 && strcmp("..", dire->d_name) && strcmp("AI", dire->d_name))
             {
                 SGPath next = path;
                 next.append(dire->d_name);
 
-                string result = fgFindAircraftPath( next, aircraft );
+                result = fgFindAircraftPath( next, aircraft );
                 if ( ! result.empty() ) {
-                    return result;
+                    break;
                 }
             }
         } else if ( !strcmp(dire->d_name, aircraft.c_str()) ) {
-            return path.str();
+            result = path.str();
+            break;
         }
     }
 
     ulCloseDir(dirp);
-
-    return "";
+    return result;
 }
 
 
@@ -607,6 +593,28 @@ bool fgInitConfig ( int argc, char **argv ) {
         SG_LOG( SG_INPUT, SG_ALERT, "No default aircraft specified" );
     }
 
+#ifdef _MSC_VER
+    char *envp = ::getenv( "APPDATA" );
+    if (envp != NULL ) {
+        SGPath config( envp );
+        config.append( "flightgear.org" );
+#else
+    if ( homedir != NULL ) {
+        SGPath config( homedir );
+        config.append( ".fgfs" );
+#endif
+        fgSetString("/sim/fg-home", config.c_str());
+        config.append( "autosave.xml" );
+        SG_LOG(SG_INPUT, SG_INFO, "Reading user settings from autosave.xml");
+        try {
+            fgLoadProps(config.str().c_str(), globals->get_props(), false,
+                        SGPropertyNode::USERARCHIVE);
+        } catch (...) {
+            SG_LOG(SG_INPUT, SG_BULK, "First time reading user settings");
+        }
+        SG_LOG(SG_INPUT, SG_BULK, "Finished Reading user settings");
+    }
+
     // parse options after loading aircraft to ensure any user
     // overrides of defaults are honored.
     do_options(argc, argv);
@@ -615,48 +623,7 @@ bool fgInitConfig ( int argc, char **argv ) {
 }
 
 
-// find basic airport location info from airport database
-bool fgFindAirportID( const string& id, FGAirport *a ) {
-    const FGAirport* result;
-    if ( id.length() ) {
-        SG_LOG( SG_GENERAL, SG_INFO, "Searching for airport code = " << id );
 
-        result = globals->get_airports()->search( id );
-
-        if ( result == NULL ) {
-            SG_LOG( SG_GENERAL, SG_ALERT,
-                    "Failed to find " << id << " in apt.dat.gz" );
-            return false;
-        }
-    } else {
-        return false;
-    }
-
-    *a = *result;
-
-    SG_LOG( SG_GENERAL, SG_INFO,
-            "Position for " << id << " is ("
-            << a->getLongitude() << ", "
-            << a->getLatitude() << ")" );
-
-    return true;
-}
-
-
-// get airport elevation
-static double fgGetAirportElev( const string& id ) {
-    FGAirport a;
-    // double lon, lat;
-
-    SG_LOG( SG_GENERAL, SG_INFO,
-            "Finding elevation for airport: " << id );
-
-    if ( fgFindAirportID( id, &a ) ) {
-        return a.getElevation();
-    } else {
-        return -9999.0;
-    }
-}
 
 
 #if 0 
@@ -666,7 +633,7 @@ static double fgGetAirportElev( const string& id ) {
 
 // Preset lon/lat given an airport id
 static bool fgSetPosFromAirportID( const string& id ) {
-    FGAirport a;
+    FGAirport *a;
     // double lon, lat;
 
     SG_LOG( SG_GENERAL, SG_INFO,
@@ -674,17 +641,17 @@ static bool fgSetPosFromAirportID( const string& id ) {
 
     if ( fgFindAirportID( id, &a ) ) {
         // presets
-        fgSetDouble("/sim/presets/longitude-deg", a.longitude );
-        fgSetDouble("/sim/presets/latitude-deg", a.latitude );
+        fgSetDouble("/sim/presets/longitude-deg", a->longitude );
+        fgSetDouble("/sim/presets/latitude-deg", a->latitude );
 
         // other code depends on the actual postition being set so set
         // that as well
-        fgSetDouble("/position/longitude-deg", a.longitude );
-        fgSetDouble("/position/latitude-deg", a.latitude );
+        fgSetDouble("/position/longitude-deg", a->longitude );
+        fgSetDouble("/position/latitude-deg", a->latitude );
 
         SG_LOG( SG_GENERAL, SG_INFO,
-                "Position for " << id << " is (" << a.longitude
-                << ", " << a.latitude << ")" );
+                "Position for " << id << " is (" << a->longitude
+                << ", " << a->latitude << ")" );
 
         return true;
     } else {
@@ -696,7 +663,7 @@ static bool fgSetPosFromAirportID( const string& id ) {
 
 // Set current tower position lon/lat given an airport id
 static bool fgSetTowerPosFromAirportID( const string& id, double hdg ) {
-    FGAirport a;
+    
     // tower height hard coded for now...
     float towerheight=50.0f;
 
@@ -704,10 +671,11 @@ static bool fgSetTowerPosFromAirportID( const string& id, double hdg ) {
     float fudge_lon = fabs(sin(hdg)) * .003f;
     float fudge_lat = .003f - fudge_lon;
 
-    if ( fgFindAirportID( id, &a ) ) {
-        fgSetDouble("/sim/tower/longitude-deg",  a.getLongitude() + fudge_lon);
-        fgSetDouble("/sim/tower/latitude-deg",  a.getLatitude() + fudge_lat);
-        fgSetDouble("/sim/tower/altitude-ft", a.getElevation() + towerheight);
+    const FGAirport *a = fgFindAirportID( id);
+    if ( a) {
+        fgSetDouble("/sim/tower/longitude-deg",  a->getLongitude() + fudge_lon);
+        fgSetDouble("/sim/tower/latitude-deg",  a->getLatitude() + fudge_lat);
+        fgSetDouble("/sim/tower/altitude-ft", a->getElevation() + towerheight);
         return true;
     } else {
         return false;
@@ -967,13 +935,13 @@ static bool fgSetPosFromNAV( const string& id, const double& freq ) {
 static bool fgSetPosFromCarrier( const string& carrier, const string& posid ) {
 
     // set initial position from runway and heading
-    Point3D geodPos;
+    SGGeod geodPos;
     double heading;
-    sgdVec3 uvw;
+    SGVec3d uvw;
     if (FGAIManager::getStartPosition(carrier, posid, geodPos, heading, uvw)) {
-        double lon = geodPos.lon() * SGD_RADIANS_TO_DEGREES;
-        double lat = geodPos.lat() * SGD_RADIANS_TO_DEGREES;
-        double alt = geodPos.elev() * SG_METER_TO_FEET;
+        double lon = geodPos.getLongitudeDeg();
+        double lat = geodPos.getLatitudeDeg();
+        double alt = geodPos.getElevationFt();
 
         SG_LOG( SG_GENERAL, SG_INFO, "Attempting to set starting position for "
                 << carrier << " at lat = " << lat << ", lon = " << lon
@@ -989,12 +957,12 @@ static bool fgSetPosFromCarrier( const string& carrier, const string& posid ) {
         fgSetDouble("/orientation/heading-deg", heading);
 
         fgSetString("/sim/presets/speed-set", "UVW");
-        fgSetDouble("/velocities/uBody-fps", uvw[0]);
-        fgSetDouble("/velocities/vBody-fps", uvw[1]);
-        fgSetDouble("/velocities/wBody-fps", uvw[2]);
-        fgSetDouble("/sim/presets/uBody-fps", uvw[0]);
-        fgSetDouble("/sim/presets/vBody-fps", uvw[1]);
-        fgSetDouble("/sim/presets/wBody-fps", uvw[2]);
+        fgSetDouble("/velocities/uBody-fps", uvw(0));
+        fgSetDouble("/velocities/vBody-fps", uvw(1));
+        fgSetDouble("/velocities/wBody-fps", uvw(2));
+        fgSetDouble("/sim/presets/uBody-fps", uvw(0));
+        fgSetDouble("/sim/presets/vBody-fps", uvw(1));
+        fgSetDouble("/sim/presets/wBody-fps", uvw(2));
 
         fgSetBool("/sim/presets/onground", true);
 
@@ -1321,7 +1289,7 @@ void fgInitFDM() {
             cur_fdm_state = new FGLaRCsim( dt );
         } else if ( model == "jsb" ) {
             cur_fdm_state = new FGJSBsim( dt );
-#ifdef ENABLE_SP_FMDS
+#if ENABLE_SP_FDMS
         } else if ( model == "ada" ) {
             cur_fdm_state = new FGADA( dt );
         } else if ( model == "acms" ) {
@@ -1568,7 +1536,6 @@ bool fgInitSubsystems() {
     //     = fgGetNode("/sim/presets/latitude-deg");
     // static const SGPropertyNode *altitude
     //     = fgGetNode("/sim/presets/altitude-ft");
-
     SG_LOG( SG_GENERAL, SG_INFO, "Initialize Subsystems");
     SG_LOG( SG_GENERAL, SG_INFO, "========== ==========");
 
@@ -1708,7 +1675,7 @@ bool fgInitSubsystems() {
     ////////////////////////////////////////////////////////////////////
 
     globals->add_subsystem("fx", new FGFX);
-    
+    globals->add_subsystem("voice", new FGVoiceMgr);
 #endif
 
     ////////////////////////////////////////////////////////////////////
@@ -1835,14 +1802,12 @@ bool fgInitSubsystems() {
     globals->get_subsystem_mgr()->bind();
     globals->get_subsystem_mgr()->init();
 
-#ifdef FG_MPLAYER_AS
     ////////////////////////////////////////////////////////////////////
     // Initialize multiplayer subsystem
     ////////////////////////////////////////////////////////////////////
 
     globals->set_multiplayer_mgr(new FGMultiplayMgr);
     globals->get_multiplayer_mgr()->init();
-#endif
 
     ////////////////////////////////////////////////////////////////////////
     // Initialize the Nasal interpreter.
@@ -1869,7 +1834,7 @@ bool fgInitSubsystems() {
                                 // Save the initial state for future
                                 // reference.
     globals->saveInitialState();
-
+    
     return true;
 }
 
