@@ -16,10 +16,13 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: fg_io.cxx,v 1.18 2004/12/29 20:40:41 curt Exp $
+// $Id: fg_io.cxx,v 1.22 2006-02-21 01:19:18 mfranz Exp $
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <simgear/compiler.h>
 
@@ -58,10 +61,7 @@
 #include <Network/ray.hxx>
 #include <Network/rul.hxx>
 #include <Network/generic.hxx>
-
-#ifdef FG_MPLAYER_AS
 #include <Network/multiplay.hxx>
-#endif
 
 #include "globals.hxx"
 #include "fg_io.hxx"
@@ -182,8 +182,6 @@ FGIO::parse_port_config( const string& config )
             else if (tokens[1] == "file") n--;
             FGGeneric *generic = new FGGeneric( tokens[n] );
             io = generic;
-
-#ifdef FG_MPLAYER_AS
 	} else if ( protocol == "multiplay" ) {\
 	    //Determine dir, rate, host & port
 	    string dir = tokens[1];
@@ -191,8 +189,6 @@ FGIO::parse_port_config( const string& config )
 	    string host = tokens[3];
 	    string port = tokens[4];
 	    return new FGMultiplay(dir, atoi(rate.c_str()), host, atoi(port.c_str()));
-#endif
-
 	} else {
 	    return NULL;
 	}
@@ -203,7 +199,11 @@ FGIO::parse_port_config( const string& config )
 	delete io;
 	return 0;
     }
-
+    
+    if (tokens.size() < 3) {
+      SG_LOG( SG_IO, SG_ALERT, "Incompatible number of network arguments.");
+      return NULL;
+    }
     string medium = tokens[1];
     SG_LOG( SG_IO, SG_INFO, "  medium = " << medium );
 
@@ -217,6 +217,10 @@ FGIO::parse_port_config( const string& config )
     SG_LOG( SG_IO, SG_INFO, "  hertz = " << hertz );
 
     if ( medium == "serial" ) {
+        if ( tokens.size() < 5) {
+          SG_LOG( SG_IO, SG_ALERT, "Incompatible number of arguments for serial communications.");
+	  return NULL;
+        }
 	// device name
 	string device = tokens[4];
 	SG_LOG( SG_IO, SG_INFO, "  device = " << device );
@@ -229,13 +233,22 @@ FGIO::parse_port_config( const string& config )
 	io->set_io_channel( ch );
     } else if ( medium == "file" ) {
 	// file name
+        if ( tokens.size() < 4) {
+          SG_LOG( SG_IO, SG_ALERT, "Incompatible number of arguments for file I/O.");
+	  return NULL;
+        }
+	  
 	string file = tokens[4];
 	SG_LOG( SG_IO, SG_INFO, "  file name = " << file );
 
 	SGFile *ch = new SGFile( file );
 	io->set_io_channel( ch );
     } else if ( medium == "socket" ) {
-	string hostname = tokens[4];
+        if ( tokens.size() < 6) {
+          SG_LOG( SG_IO, SG_ALERT, "Incompatible number of arguments for socket communications.");
+	  return NULL;
+        }
+      	string hostname = tokens[4];
 	string port = tokens[5];
 	string style = tokens[6];
 
