@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: globals.cxx,v 1.38 2005/10/22 11:22:06 ehofman Exp $
+// $Id: globals.cxx,v 1.43 2006-02-21 01:19:18 mfranz Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -117,7 +117,9 @@ void FGGlobals::set_fg_root (const string &root) {
     tmp.append( "version" );
     if ( ulFileExists( tmp.c_str() ) ) {
         fg_root += "/data";
-        }
+    }
+
+    fgSetString("/sim/fg-root", fg_root.c_str());   
 }
 
 void FGGlobals::set_fg_scenery (const string &scenery) {
@@ -157,6 +159,10 @@ void FGGlobals::set_fg_scenery (const string &scenery) {
                 ulCloseDir( od );
             }
         }
+        // insert a marker for FGTileEntry::load(), so that
+        // FG_SCENERY=A:B becomes list ["A/Terrain", "A/Objects", "",
+        // "B/Terrain", "B/Objects", ""]
+        fg_scenery.push_back("");
     }
 }
 
