@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: main.cxx,v 1.213 2005/11/17 22:54:46 curt Exp $
+// $Id: main.cxx,v 1.218 2006-03-21 18:52:19 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -74,12 +74,7 @@
 #include <Time/fg_timer.hxx>
 #include <Environment/environment_mgr.hxx>
 #include <GUI/new_gui.hxx>
-
-#ifdef FG_MPLAYER_AS
 #include <MultiPlayer/multiplaymgr.hxx>
-#endif
-
-
 
 #include "fg_commands.hxx"
 #include "fg_io.hxx"
@@ -220,6 +215,8 @@ static void fgMainLoop( void ) {
         = fgGetNode("/sim/freeze/clock", true);
     static const SGPropertyNode *cur_time_override
         = fgGetNode("/sim/time/cur-time-override", true);
+    static const SGPropertyNode *max_simtime_per_frame
+        = fgGetNode("/sim/max-simtime-per-frame", true);
 
     SGCloudLayer::enable_bump_mapping = fgGetBool("/sim/rendering/bump-mapping");
 
@@ -292,6 +289,20 @@ static void fgMainLoop( void ) {
 
     real_delta_time_sec
         = double(current_time_stamp - last_time_stamp) / 1000000.0;
+
+    // Limit the time we need to spend in simulation loops
+    // That means, if the /sim/max-simtime-per-frame value is strictly positive
+    // you can limit the maximum amount of time you will do simulations for
+    // one frame to display. The cpu time spent in simulations code is roughtly
+    // at least O(real_delta_time_sec). If this is (due to running debug
+    // builds or valgrind or something different blowing up execution times)
+    // larger than the real time you will no more get any response
+    // from flightgear. This limits that effect. Just set to property from
+    // your .fgfsrc or commandline ...
+    double dtMax = max_simtime_per_frame->getDoubleValue();
+    if (0 < dtMax && dtMax < real_delta_time_sec)
+      real_delta_time_sec = dtMax;
+
     // round the real time down to a multiple of 1/model-hz.
     // this way all systems are updated the _same_ amount of dt.
     {
@@ -438,19 +449,20 @@ static void fgMainLoop( void ) {
     ++frames;
 #endif
 
+    // Update any multiplayer's network queues, the AIMultiplayer
+    // implementation is an AI model and depends on that
+    globals->get_multiplayer_mgr()->Update();
+
     // Run ATC subsystem
     if (fgGetBool("/sim/atc/enabled"))
         globals->get_ATC_mgr()->update(delta_time_sec);
 
     // Run the AI subsystem
+    // FIXME: run that also if we have multiplying enabled since the
+    // multiplayer information is interpreted by an AI model
     if (fgGetBool("/sim/ai-traffic/enabled"))
         globals->get_AI_mgr()->update(delta_time_sec);
 
-#ifdef FG_MPLAYER_AS
-    // Update any multiplayer models
-    globals->get_multiplayer_mgr()->Update();
-#endif
-
     // Run flight model
 
     // Calculate model iterations needed for next frame
@@ -900,6 +912,23 @@ static void fgIdleFunction ( void ) {
 }
 
 
+static void upper_case_property(const char *name)
+{
+    SGPropertyNode *p = fgGetNode(name, false);
+    if (!p) {
+        p = fgGetNode(name, true);
+        p->setStringValue("");
+    } else {
+        SGPropertyNode::Type t = p->getType();
+        if (t == SGPropertyNode::NONE || t == SGPropertyNode::UNSPECIFIED)
+            p->setStringValue("");
+        else
+            assert(t == SGPropertyNode::STRING);
+    }
+    p->addChangeListener(new FGMakeUpperCase);
+}
+
+
 // Main top level initialization
 bool fgMainInit( int argc, char **argv ) {
 
@@ -936,12 +965,16 @@ bool fgMainInit( int argc, char **argv ) {
     string_list *col = new string_list;
     globals->set_channel_options_list( col );
 
+    upper_case_property("/sim/presets/airport-id");
+    upper_case_property("/sim/presets/runway");
+    upper_case_property("/sim/tower/airport-id");
+
     // Scan the config file(s) and command line options to see if
     // fg_root was specified (ignore all other options for now)
     fgInitFGRoot(argc, argv);
 
     // Check for the correct base package version
-    static char required_version[] = "0.9.9";
+    static char required_version[] = "0.9.10";
     string base_version = fgBasePackageVersion();
     if ( !(base_version == required_version) ) {
         // tell the operator how to use this application
