@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: options.cxx,v 1.76 2005/11/13 10:20:15 ehofman Exp $
+// $Id: options.cxx,v 1.82 2006-03-21 14:53:38 fredb Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -230,14 +230,11 @@ fgSetDefaults ()
     fgSetBool("/sim/freeze/clock", false);
     fgSetBool("/sim/freeze/fuel", false);
 
-#ifdef FG_MPLAYER_AS
     fgSetString("/sim/multiplay/callsign", "callsign");
     fgSetString("/sim/multiplay/rxhost", "0");
     fgSetString("/sim/multiplay/txhost", "0");
     fgSetInt("/sim/multiplay/rxport", 0);
     fgSetInt("/sim/multiplay/txport", 0);
-#endif
-
 }
 
 static bool
@@ -1063,7 +1060,7 @@ fgOptConfig( const char *arg )
 	readProperties(file, globals->get_props());
     } catch (const sg_exception &e) {
 	string message = "Error loading config file: ";
-	message += e.getFormattedMessage();
+	message += e.getFormattedMessage() + e.getOrigin();
 	SG_LOG(SG_INPUT, SG_ALERT, message);
 	exit(2);
     }
@@ -1297,6 +1294,8 @@ struct OptionDesc {
     {"aspect-ratio-multiplier",      true,  OPTION_DOUBLE, "/sim/current-view/aspect-ratio-multiplier", false, "", 0 },
     {"disable-fullscreen",           false, OPTION_BOOL,   "/sim/startup/fullscreen", false, "", 0 },
     {"enable-fullscreen",            false, OPTION_BOOL,   "/sim/startup/fullscreen", true, "", 0 },
+    {"disable-save-on-exit",         false, OPTION_BOOL,   "/sim/startup/save-on-exit", false, "", 0 },
+    {"enable-save-on-exit",          false, OPTION_BOOL,   "/sim/startup/save-on-exit", true, "", 0 },
     {"shading-flat",                 false, OPTION_BOOL,   "/sim/rendering/shading", false, "", 0 },
     {"shading-smooth",               false, OPTION_BOOL,   "/sim/rendering/shading", true, "", 0 },
     {"disable-skyblend",             false, OPTION_BOOL,   "/sim/rendering/skyblend", false, "", 0 },
@@ -1310,6 +1309,7 @@ struct OptionDesc {
     {"units-feet",                   false, OPTION_STRING, "/sim/startup/units", false, "feet", 0 },
     {"units-meters",                 false, OPTION_STRING, "/sim/startup/units", false, "meters", 0 },
     {"timeofday",                    true,  OPTION_STRING, "/sim/startup/time-offset-type", false, "noon", 0 },
+    {"season",                       true,  OPTION_STRING, "/sim/startup/season", false, "summer", 0 },
     {"time-offset",                  true,  OPTION_FUNC,   "", false, "", fgOptTimeOffset },
     {"time-match-real",              false, OPTION_STRING, "/sim/startup/time-offset-type", false, "system-offset", 0 },
     {"time-match-local",             false, OPTION_STRING, "/sim/startup/time-offset-type", false, "latitude-offset", 0 },
@@ -1340,10 +1340,8 @@ struct OptionDesc {
     {"joyclient",                    true,  OPTION_CHANNEL, "", false, "", 0 },
     {"jsclient",                     true,  OPTION_CHANNEL, "", false, "", 0 },
     {"proxy",                        true,  OPTION_FUNC,    "", false, "", fgSetupProxy },
-#ifdef FG_MPLAYER_AS
     {"callsign",                     true, OPTION_STRING,  "sim/multiplay/callsign", false, "", 0 },
     {"multiplay",                    true,  OPTION_CHANNEL, "", false, "", 0 },
-#endif
     {"trace-read",                   true,  OPTION_FUNC,   "", false, "", fgOptTraceRead },
     {"trace-write",                  true,  OPTION_FUNC,   "", false, "", fgOptTraceWrite },
     {"log-level",                    true,  OPTION_FUNC,   "", false, "", fgOptLogLevel },
@@ -1397,15 +1395,14 @@ parse_option (const string& arg)
     } else if ( arg.find( "--prop:" ) == 0 ) {
         string assign = arg.substr(7);
 	string::size_type pos = assign.find('=');
-	if ( pos == arg.npos || pos == 0 ) {
+	if ( pos == arg.npos || pos == 0 || pos + 1 == assign.size() ) {
 	    SG_LOG( SG_GENERAL, SG_ALERT, "Bad property assignment: " << arg );
-	    return FG_OPTIONS_ERROR;
-	}
-	string name = assign.substr(0, pos);
-	string value = assign.substr(pos + 1);
-	fgSetString(name.c_str(), value.c_str());
-	// SG_LOG(SG_GENERAL, SG_INFO, "Setting default value of property "
-	//        << name << " to \"" << value << '"');
+            return FG_OPTIONS_ERROR;
+        } else {
+	    string name = assign.substr(0, pos);
+	    string value = assign.substr(pos + 1);
+	    fgSetString(name.c_str(), value.c_str());
+        }
     } else if ( arg.find( "--" ) == 0 ) {
         size_t pos = arg.find( '=' );
         string arg_name;
@@ -1422,7 +1419,7 @@ parse_option (const string& arg)
                     fgSetBool( pt->property, pt->b_param );
                     break;
                 case OPTION_STRING:
-                    if ( pt->has_param && pos != string::npos ) {
+                    if ( pt->has_param && pos != string::npos && pos + 1 < arg.size() ) {
                         fgSetString( pt->property, arg.substr( pos + 1 ).c_str() );
                     } else if ( !pt->has_param && pos == string::npos ) {
                         fgSetString( pt->property, pt->s_param );
@@ -1435,7 +1432,7 @@ parse_option (const string& arg)
                     }
                     break;
                 case OPTION_DOUBLE:
-                    if ( pos != string::npos ) {
+                    if ( pos != string::npos && pos + 1 < arg.size() ) {
                         fgSetDouble( pt->property, atof( arg.substr( pos + 1 ) ) );
                     } else {
                         SG_LOG( SG_GENERAL, SG_ALERT, "Option '" << arg << "' needs a parameter" );
@@ -1443,7 +1440,7 @@ parse_option (const string& arg)
                     }
                     break;
                 case OPTION_INT:
-                    if ( pos != string::npos ) {
+                    if ( pos != string::npos && pos + 1 < arg.size() ) {
                         fgSetInt( pt->property, atoi( arg.substr( pos + 1 ) ) );
                     } else {
                         SG_LOG( SG_GENERAL, SG_ALERT, "Option '" << arg << "' needs a parameter" );
@@ -1451,7 +1448,7 @@ parse_option (const string& arg)
                     }
                     break;
                 case OPTION_CHANNEL:
-                    if ( pt->has_param && pos != string::npos ) {
+                    if ( pt->has_param && pos != string::npos && pos + 1 < arg.size() ) {
                         add_channel( pt->option, arg.substr( pos + 1 ) );
                     } else if ( !pt->has_param && pos == string::npos ) {
                         add_channel( pt->option, pt->s_param );
@@ -1464,7 +1461,7 @@ parse_option (const string& arg)
                     }
                     break;
                 case OPTION_FUNC:
-                    if ( pt->has_param && pos != string::npos ) {
+                    if ( pt->has_param && pos != string::npos && pos + 1 < arg.size() ) {
                         return pt->func( arg.substr( pos + 1 ).c_str() );
                     } else if ( !pt->has_param && pos == string::npos ) {
                         return pt->func( 0 );
