@@ -17,9 +17,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: renderer.cxx,v 1.31 2005/11/08 10:00:24 ehofman Exp $
+// $Id: renderer.cxx,v 1.46 2006-04-01 10:50:52 frohlich Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -74,12 +74,9 @@
 #include "main.hxx"
 
 
-float default_attenuation[3] = {1.0, 0.0, 0.0};
+extern void sgShaderFrameInit(double delta_time_sec);
 
-ssgSelector *lightpoints_brightness = new ssgSelector;
-ssgTransform *lightpoints_transform = new ssgTransform;
-FGTileEntry *dummy_tile;
-sgVec3 rway_ols;
+float default_attenuation[3] = {1.0, 0.0, 0.0};
 
 // Clip plane settings...
 float scene_nearplane = 0.5f;
@@ -88,6 +85,7 @@ float scene_farplane = 120000.0f;
 glPointParameterfProc glPointParameterfPtr = 0;
 glPointParameterfvProc glPointParameterfvPtr = 0;
 bool glPointParameterIsSupported = false;
+bool glPointSpriteIsSupported = false;
 
 
 // fog constants.  I'm a little nervous about putting actual code out
@@ -103,19 +101,9 @@ static GLfloat ground_exp2_punch_through;
 // Sky structures
 SGSky *thesky;
 
-// hack
-sgMat4 copy_of_ssgOpenGLAxisSwapMatrix =
-{
-  {  1.0f,  0.0f,  0.0f,  0.0f },
-  {  0.0f,  0.0f, -1.0f,  0.0f },
-  {  0.0f,  1.0f,  0.0f,  0.0f },
-  {  0.0f,  0.0f,  0.0f,  1.0f }
-};
-
-ssgSimpleState *cloud3d_imposter_state;
-ssgSimpleState *default_state;
-ssgSimpleState *hud_and_panel;
-ssgSimpleState *menus;
+ssgSharedPtr<ssgSimpleState> default_state;
+ssgSharedPtr<ssgSimpleState> hud_and_panel;
+ssgSharedPtr<ssgSimpleState> menus;
 
 SGShadowVolume *shadows;
 
@@ -137,7 +125,6 @@ FGRenderer::~FGRenderer()
 void
 FGRenderer::build_states( void ) {
     default_state = new ssgSimpleState;
-    default_state->ref();
     default_state->disable( GL_TEXTURE_2D );
     default_state->enable( GL_CULL_FACE );
     default_state->enable( GL_COLOR_MATERIAL );
@@ -148,29 +135,13 @@ FGRenderer::build_states( void ) {
     default_state->disable( GL_ALPHA_TEST );
     default_state->disable( GL_LIGHTING );
 
-    cloud3d_imposter_state = new ssgSimpleState;
-    cloud3d_imposter_state->ref();
-    cloud3d_imposter_state->enable( GL_TEXTURE_2D );
-    cloud3d_imposter_state->enable( GL_CULL_FACE );
-    cloud3d_imposter_state->enable( GL_COLOR_MATERIAL );
-    cloud3d_imposter_state->setColourMaterial( GL_AMBIENT_AND_DIFFUSE );
-    cloud3d_imposter_state->setMaterial( GL_DIFFUSE, 1, 1, 1, 1 );
-    cloud3d_imposter_state->setMaterial( GL_AMBIENT, 1, 1, 1, 1 );
-    cloud3d_imposter_state->setMaterial( GL_EMISSION, 0, 0, 0, 1 );
-    cloud3d_imposter_state->setMaterial( GL_SPECULAR, 0, 0, 0, 1 );
-    cloud3d_imposter_state->enable( GL_BLEND );
-    cloud3d_imposter_state->enable( GL_ALPHA_TEST );
-    cloud3d_imposter_state->disable( GL_LIGHTING );
-
     hud_and_panel = new ssgSimpleState;
-    hud_and_panel->ref();
     hud_and_panel->disable( GL_CULL_FACE );
     hud_and_panel->disable( GL_TEXTURE_2D );
     hud_and_panel->disable( GL_LIGHTING );
     hud_and_panel->enable( GL_BLEND );
 
     menus = new ssgSimpleState;
-    menus->ref();
     menus->disable( GL_CULL_FACE );
     menus->disable( GL_TEXTURE_2D );
     menus->enable( GL_BLEND );
@@ -212,7 +183,7 @@ FGRenderer::init( void ) {
         // if fastest fog requested, or if flat shading force fastest
         glHint ( GL_FOG_HINT, GL_FASTEST );
     } else if ( !strcmp(fgGetString("/sim/rendering/fog"), "nicest") ) {
-        glHint ( GL_FOG_HINT, GL_NICEST );
+        glHint ( GL_FOG_HINT, GL_DONT_CARE );
     }
     if ( fgGetBool("/sim/rendering/wireframe") ) {
         // draw wire frame
@@ -223,9 +194,21 @@ FGRenderer::init( void ) {
     glFrontFace ( GL_CCW );
 
     // Just testing ...
-    // glEnable(GL_POINT_SMOOTH);
-    // glEnable(GL_LINE_SMOOTH);
+    if ( SGIsOpenGLExtensionSupported("GL_ARB_point_sprite") ||
+         SGIsOpenGLExtensionSupported("GL_NV_point_sprite") )
+    {
+        GLuint handle = thesky->get_sun_texture_id();
+        glBindTexture ( GL_TEXTURE_2D, handle ) ;
+        glTexEnvf(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);
+        glEnable(GL_POINT_SPRITE);
+        // glEnable(GL_POINT_SMOOTH);
+        glPointSpriteIsSupported = true;
+    }
+    glEnable(GL_LINE_SMOOTH);
     // glEnable(GL_POLYGON_SMOOTH);      
+    glHint(GL_POLYGON_SMOOTH_HINT, GL_DONT_CARE);
+    glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
+    glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
 }
 
 
@@ -233,7 +216,8 @@ FGRenderer::init( void ) {
 // Update all Visuals (redraws anything graphics related)
 void
 FGRenderer::update( bool refresh_camera_settings ) {
-    bool scenery_loaded = fgGetBool("sim/sceneryloaded") || fgGetBool("sim/sceneryloaded-override");
+    bool scenery_loaded = fgGetBool("sim/sceneryloaded") \
+                          || fgGetBool("sim/sceneryloaded-override");
 
     if ( idle_state < 1000 || !scenery_loaded ) {
         // still initializing, draw the splash screen
@@ -244,13 +228,6 @@ FGRenderer::update( bool refresh_camera_settings ) {
         SGAnimation::set_sim_time_sec( 0.0 );
         return;
     }
-//    return; 
-
-	// TODO:TEST only, don't commit that !!
-//	sgFXperFrameInit();
-
-    extern void sgShaderFrameInit(double delta_time_sec);
-    sgShaderFrameInit(delta_time_sec);
 
     bool draw_otw = fgGetBool("/sim/rendering/draw-otw");
     bool skyblend = fgGetBool("/sim/rendering/skyblend");
@@ -289,6 +266,11 @@ FGRenderer::update( bool refresh_camera_settings ) {
         actual_visibility = fgGetDouble("/environment/visibility-m");
     }
 
+        // TODO:TEST only, don't commit that !!
+//      sgFXperFrameInit();
+
+    sgShaderFrameInit(delta_time_sec);
+
     if ( actual_visibility != last_visibility ) {
         last_visibility = actual_visibility;
 
@@ -695,14 +677,20 @@ FGRenderer::update( bool refresh_camera_settings ) {
             glDepthMask( GL_TRUE );
         }
     }
+       double current_view_origin_airspeed_horiz_kt =
+        fgGetDouble("/velocities/airspeed-kt", 0.0)
+                       * cos( fgGetDouble("/orientation/pitch-deg", 0.0)
+                               * SGD_DEGREES_TO_RADIANS);
+       // TODO:find the real view speed, not the AC one
     sgEnviro.drawPrecipitation(
         fgGetDouble("/environment/metar/rain-norm", 0.0),
         fgGetDouble("/environment/metar/snow-norm", 0.0),
         fgGetDouble("/environment/metar/hail-norm", 0.0),
-        current__view->getPitch_deg() - current__view->getPitchOffset_deg(),
+        current__view->getPitch_deg() + current__view->getPitchOffset_deg(),
         current__view->getRoll_deg() + current__view->getRollOffset_deg(),
         - current__view->getHeadingOffset_deg(),
-        fgGetDouble("/velocities/airspeed-kt", 0.0));
+               current_view_origin_airspeed_horiz_kt
+               );
 
     // compute shadows and project them on screen
     bool is_internal = globals->get_current_view()->getInternal();
@@ -785,8 +773,13 @@ FGRenderer::resize( int width, int height ) {
 
     glViewport( 0, (GLint)(height - view_h), (GLint)(width), (GLint)(view_h) );
 
-    fgSetInt("/sim/startup/xsize", width);
-    fgSetInt("/sim/startup/ysize", height);
+    static int lastwidth = width;
+    static int lastheight = height;
+    if (width != lastwidth)
+        fgSetInt("/sim/startup/xsize", lastwidth = width);
+    if (height != lastheight)
+        fgSetInt("/sim/startup/ysize", lastheight = height);
+
     guiInitMouse(width, height);
 
     // for all views
@@ -920,5 +913,50 @@ void FGRenderer::setNearFar( float n, float f ) {
     fgHackFrustum();
 }
 
+bool FGRenderer::getPickInfo( sgdVec3 pt, sgdVec3 dir, unsigned x, unsigned y )
+{
+  // Get the matrices involved in the transform from global to screen
+  // coordinates.
+  sgMat4 pm;
+  ssgGetProjectionMatrix(pm);
+  sgMat4 mv;
+  ssgGetModelviewMatrix(mv);
+  
+  // Compose ...
+  sgMat4 m;
+  sgMultMat4(m, pm, mv);
+  // ... and invert
+  sgInvertMat4(m);
+  
+  // Get the width and height of the display to be able to normalize the
+  // mouse coordinate
+  float width = fgGetInt("/sim/startup/xsize");
+  float height = fgGetInt("/sim/startup/ysize");
+  
+  // Compute some coordinates of in the line from the eyepoint to the
+  // mouse click coodinates.
+  // First build the normalized projection coordinates
+  sgVec4 normPt;
+  sgSetVec4(normPt, (2*x - width)/width, -(2*y - height)/height, 1, 1);
+  // Transform them into the real world
+  sgVec4 worldPt;
+  sgXformPnt4(worldPt, normPt, m);
+  if (worldPt[3] == 0)
+    return false;
+  sgScaleVec3(worldPt, 1/worldPt[3]);
+
+  // Now build a direction from the point
+  FGViewer* view = globals->get_current_view();
+  sgVec4 fDir;
+  sgSubVec3(fDir, worldPt, view->get_view_pos());
+  sgdSetVec3(dir, fDir);
+  sgdNormalizeVec3(dir);
+
+  // Copy the start point
+  sgdCopyVec3(pt, view->get_absolute_view_pos());
+
+  return true;
+}
 
 // end of renderer.cxx
+    
