@@ -17,9 +17,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: splash.cxx,v 1.17 2005/07/07 21:43:22 mfranz Exp $
+// $Id: splash.cxx,v 1.21 2006-02-21 01:19:20 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -52,6 +52,7 @@
 #include "fg_props.hxx"
 #include "splash.hxx"
 #include "fg_os.hxx"
+#include "renderer.hxx"
 
 static const int fontsize = 19;
 static const char fontname[] = "default.txf";
@@ -142,6 +143,7 @@ void fgSplashUpdate ( float alpha ) {
     if (!screen_width || !screen_height)
         return;
 
+    globals->get_renderer()->resize(screen_width, screen_height);
     int size = screen_width < (screen_height - 5 * fontsize)
             ? screen_width : screen_height - 5 * fontsize;
     if (size > 512)
@@ -166,7 +168,9 @@ void fgSplashUpdate ( float alpha ) {
     glDisable(GL_LIGHTING);
 
     // draw the background
-    glColor4f( 0.0, 0.0, 0.0, alpha );
+    FGColor c(0.0, 0.0, 0.0);
+    c.merge(fgGetNode("/sim/gui/colors/splash-screen"));
+    glColor4f(c.red(), c.green(), c.blue(), alpha );
     glBegin(GL_POLYGON);
     glVertex2f(0.0, 0.0);
     glVertex2f(screen_width, 0.0);
@@ -200,7 +204,7 @@ void fgSplashUpdate ( float alpha ) {
 
         info.begin();
         FGColor c(1.0, 0.9, 0.0);
-        c.merge(fgGetNode("/sim/gui/colors/splash"));
+        c.merge(fgGetNode("/sim/gui/colors/splash-font"));
         glColor4f(c.red(), c.green(), c.blue(), alpha);
 
         font.getBBox(progress_text, fontsize, 0, &left, &right, &bot, &top);
