@@ -17,9 +17,13 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: viewmgr.cxx,v 1.15 2005/04/29 14:38:24 ehofman Exp $
+// $Id: viewmgr.cxx,v 1.17 2006-02-21 01:19:21 mfranz Exp $
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <string.h>		// strcmp
 
