@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: fixlist.cxx,v 1.4 2004/11/19 22:10:43 curt Exp $
+// $Id: fixlist.cxx,v 1.8 2006-02-21 01:19:37 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -32,9 +32,6 @@
 #include "fixlist.hxx"
 
 
-FGFixList *current_fixlist;
-
-
 // Constructor
 FGFixList::FGFixList( void ) {
 }
@@ -47,7 +44,6 @@ FGFixList::~FGFixList( void ) {
 
 // load the navaids and build the map
 bool FGFixList::init( SGPath path ) {
-
     fixlist.erase( fixlist.begin(), fixlist.end() );
 
     sg_gzifstream in( path.str() );
@@ -83,7 +79,6 @@ bool FGFixList::init( SGPath path ) {
         fixlist[fix.get_ident()] = fix;
         in >> skipcomment;
     }
-    
     return true;
 }
 
@@ -120,3 +115,18 @@ bool FGFixList::query_and_offset( const string& ident, double lon, double lat,
     *dist = s;
     return true;
 }
+
+const FGFix* FGFixList::findFirstByIdent( const string& ident, bool exact)
+{
+    fix_map_iterator itr;
+    if(exact) {
+        itr = fixlist.find(ident);
+    } else {
+        itr = fixlist.lower_bound(ident);
+    }
+    if(itr == fixlist.end()) {
+        return(NULL);
+    } else {
+        return(&(itr->second));
+    }
+}
