@@ -16,10 +16,13 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: generic.cxx,v 1.14 2004/11/19 22:10:43 curt Exp $
+// $Id: generic.cxx,v 1.17 2006-03-09 21:41:51 mfranz Exp $
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <string.h>		// strstr()
 #include <stdlib.h>		// strtod(), atoi()
@@ -130,7 +133,7 @@ bool FGGeneric::parse_message() {
     double val;
     int i = -1;
 
-    while ((++i < _in_message.size()) &&
+    while ((++i < (int)_in_message.size()) &&
            p1 && strcmp(p1, line_separator.c_str())) {
 
         p2 = strstr(p1, var_separator.c_str());
