@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: native_fdm.cxx,v 1.29 2005/08/14 12:57:13 ehofman Exp $
+// $Id: native_fdm.cxx,v 1.31 2006-03-21 18:51:57 curt Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -171,6 +171,7 @@ void FGProps2NetFDM( FGNetFDM *net, bool net_byte_order ) {
         }
         net->rpm[i] = node->getDoubleValue( "rpm" );
         net->fuel_flow[i] = node->getDoubleValue( "fuel-flow-gph" );
+        net->fuel_px[i] = node->getDoubleValue( "fuel-px-psi" );
         net->egt[i] = node->getDoubleValue( "egt-degf" );
         // cout << "egt = " << aero->EGT << endl;
         net->cht[i] = node->getDoubleValue( "cht-degf" );
@@ -254,6 +255,7 @@ void FGProps2NetFDM( FGNetFDM *net, bool net_byte_order ) {
             net->eng_state[i] = htonl(net->eng_state[i]);
             htonf(net->rpm[i]);
             htonf(net->fuel_flow[i]);
+            htonf(net->fuel_px[i]);
             htonf(net->egt[i]);
             htonf(net->cht[i]);
             htonf(net->mp_osi[i]);
@@ -335,6 +337,7 @@ void FGNetFDM2Props( FGNetFDM *net, bool net_byte_order ) {
             net->eng_state[i] = htonl(net->eng_state[i]);
             htonf(net->rpm[i]);
             htonf(net->fuel_flow[i]);
+            htonf(net->fuel_px[i]);
             htonf(net->egt[i]);
             htonf(net->cht[i]);
             htonf(net->mp_osi[i]);
@@ -433,6 +436,7 @@ void FGNetFDM2Props( FGNetFDM *net, bool net_byte_order ) {
 
 	    node->setDoubleValue( "rpm", net->rpm[i] );
 	    node->setDoubleValue( "fuel-flow-gph", net->fuel_flow[i] );
+	    node->setDoubleValue( "fuel-px-psi", net->fuel_px[i] );
 	    node->setDoubleValue( "egt-degf", net->egt[i] );
 	    node->setDoubleValue( "cht-degf", net->cht[i] );
 	    node->setDoubleValue( "mp-osi", net->mp_osi[i] );
