@@ -18,9 +18,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: props.cxx,v 1.16 2004/11/19 22:10:43 curt Exp $
+// $Id: props.cxx,v 1.18 2006-02-21 01:19:47 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -354,6 +354,16 @@ PropsChannel::foundTerminator()
                             = args.getNode("timeofday", i-2, true);
                         node->setStringValue( tokens[i].c_str() );
                     }
+                } else if ( tokens[1] == "play-audio-message" ) {
+                    if ( tokens.size() == 4 ) {
+                        cout << "props: play audio message = " << tokens[2]
+                             << " " << tokens[3] << endl;
+                        SGPropertyNode *node;
+                        node = args.getNode("path", 0, true);
+                        node->setStringValue( tokens[2].c_str() );
+                        node = args.getNode("file", 0, true);
+                        node->setStringValue( tokens[3].c_str() );
+                   }
                 }
                 if ( !globals->get_commands()
                          ->execute(tokens[1].c_str(), &args) )
