@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: scenery.cxx,v 1.12 2005/11/09 17:08:04 curt Exp $
+// $Id: scenery.cxx,v 1.14 2006-02-21 01:19:56 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -112,7 +112,6 @@ void FGScenery::set_center( const Point3D& p ) {
 }
 
 void FGScenery::register_placement_transform(ssgPlacementTransform *trans) {
-    trans->ref();
     _placement_list.push_back(trans);        
     sgdVec3 c;
     sgdSetVec3(c, center.x(), center.y(), center.z());
@@ -124,7 +123,6 @@ void FGScenery::unregister_placement_transform(ssgPlacementTransform *trans) {
     while (it != _placement_list.end()) {
         if ((*it) == trans) {
             it = _placement_list.erase(it);        
-            ssgDeRefDelete(trans);
         } else
             ++it;
     }
@@ -155,7 +153,7 @@ FGScenery::get_cart_elevation_m(const sgdVec3& pos, double max_altoff,
     Point3D ppos(pos[0], pos[1], pos[2]);
     if (30.0*30.0 < ppos.distance3Dsquared(center)) {
       set_center( ppos );
-      replaced_center = false;
+      replaced_center = true;
     }
   }
 
