@@ -16,9 +16,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: tileentry.cxx,v 1.47 2005/10/14 16:25:14 ehofman Exp $
+// $Id: tileentry.cxx,v 1.52 2006-03-09 09:04:04 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -201,16 +201,6 @@ static void my_remove_branch( ssgBranch * branch ) {
 // is intended to spread the load of freeing a complex tile out over
 // several frames.
 static int fgPartialFreeSSGtree( ssgBranch *b, int n ) {
-
-#if 0
-    // for testing: we could call the following two lines and replace
-    // the functionality of this entire function and everything will
-    // get properly freed, but it will happen all at once and could
-    // cause a huge frame rate hit.
-    ssgDeRefDelete( b );
-    return 0;
-#endif
-
     int num_deletes = 0;
 
     if ( n > 0 ) {
@@ -264,7 +254,7 @@ bool FGTileEntry::free_tile() {
         // disconnected from the scene graph)
         SG_LOG( SG_TERRAIN, SG_DEBUG, "FREEING terra_transform" );
         if ( fgPartialFreeSSGtree( terra_transform, delete_size ) == 0 ) {
-            ssgDeRefDelete( terra_transform );
+            terra_transform = 0;
             free_tracker |= TERRA_NODE;
         }
     } else if ( !(free_tracker & GROUND_LIGHTS) && gnd_lights_transform ) {
@@ -272,7 +262,7 @@ bool FGTileEntry::free_tile() {
         // disconnected from the scene graph)
         SG_LOG( SG_TERRAIN, SG_DEBUG, "FREEING gnd_lights_transform" );
         if ( fgPartialFreeSSGtree( gnd_lights_transform, delete_size ) == 0 ) {
-            ssgDeRefDelete( gnd_lights_transform );
+            gnd_lights_transform = 0;
             free_tracker |= GROUND_LIGHTS;
         }
     } else if ( !(free_tracker & VASI_LIGHTS) && vasi_lights_selector ) {
@@ -280,7 +270,7 @@ bool FGTileEntry::free_tile() {
         // been disconnected from the scene graph)
         SG_LOG( SG_TERRAIN, SG_DEBUG, "FREEING vasi_lights_selector" );
         if ( fgPartialFreeSSGtree( vasi_lights_selector, delete_size ) == 0 ) {
-            ssgDeRefDelete( vasi_lights_selector );
+            vasi_lights_selector = 0;
             free_tracker |= VASI_LIGHTS;
         }
     } else if ( !(free_tracker & RWY_LIGHTS) && rwy_lights_selector ) {
@@ -288,7 +278,7 @@ bool FGTileEntry::free_tile() {
         // been disconnected from the scene graph)
         SG_LOG( SG_TERRAIN, SG_DEBUG, "FREEING rwy_lights_selector" );
         if ( fgPartialFreeSSGtree( rwy_lights_selector, delete_size ) == 0 ) {
-            ssgDeRefDelete( rwy_lights_selector );
+            rwy_lights_selector = 0;
             free_tracker |= RWY_LIGHTS;
         }
     } else if ( !(free_tracker & TAXI_LIGHTS) && taxi_lights_selector ) {
@@ -296,7 +286,7 @@ bool FGTileEntry::free_tile() {
         // disconnected from the scene graph)
         SG_LOG( SG_TERRAIN, SG_DEBUG, "FREEING taxi_lights_selector" );
         if ( fgPartialFreeSSGtree( taxi_lights_selector, delete_size ) == 0 ) {
-            ssgDeRefDelete( taxi_lights_selector );
+            taxi_lights_selector = 0;
             free_tracker |= TAXI_LIGHTS;
         }
     } else if ( !(free_tracker & LIGHTMAPS) ) {
@@ -659,276 +649,266 @@ bool FGTileEntry::obj_load( const string& path,
 }
 
 
+typedef enum {
+    OBJECT,
+    OBJECT_SHARED,
+    OBJECT_STATIC,
+    OBJECT_TAXI_SIGN,
+    OBJECT_RUNWAY_SIGN
+} object_type;
+
+
+// storage class for deferred object processing in FGTileEntry::load()
+struct Object {
+    Object(object_type t, const string& token, const SGPath& p, istream& in)
+        : type(t), path(p)
+    {
+        in >> name;
+        if (type != OBJECT)
+            in >> lon >> lat >> elev >> hdg;
+        in >> ::skipeol;
+
+        if (type == OBJECT)
+            SG_LOG(SG_TERRAIN, SG_INFO, "    " << token << "  " << name);
+        else
+            SG_LOG(SG_TERRAIN, SG_INFO, "    " << token << "  " << name << "  lon=" <<
+                    lon << "  lat=" << lat << "  elev=" << elev << "  hdg=" << hdg);
+    }
+    object_type type;
+    string name;
+    SGPath path;
+    double lon, lat, elev, hdg;
+};
+
+
 void
 FGTileEntry::load( const string_list &path_list, bool is_base )
 {
     bool found_tile_base = false;
 
-    // obj_load() will generate ground lighting for us ...
-    ssgVertexArray *light_pts = new ssgVertexArray( 100 );
+    SGPath object_base;
+    vector<const Object*> objects;
 
-    ssgBranch* new_tile = new ssgBranch;
+    string index_str = tile_bucket.gen_index_str();
+    SG_LOG( SG_TERRAIN, SG_INFO, "Loading tile " << index_str );
 
-    unsigned int i = 0;
-    while ( i < path_list.size() ) {
+    // scan and parse all files and store information
+    for (unsigned int i = 0; i < path_list.size(); i++) {
+        // If we found a terrain tile in Terrain/, we have to process the
+        // Objects/ dir in the same group, too, before we can stop scanning.
+        // FGGlobals::set_fg_scenery() inserts an empty string to path_list
+        // as marker.
+        if (path_list[i].empty()) {
+            if (found_tile_base)
+                break;
+            else
+                continue;
+        }
 
         bool has_base = false;
 
-        // Generate names for later use
-        string index_str = tile_bucket.gen_index_str();
-
         SGPath tile_path = path_list[i];
         tile_path.append( tile_bucket.gen_base_path() );
 
         SGPath basename = tile_path;
         basename.append( index_str );
-        // string path = basename.str();
 
-        SG_LOG( SG_TERRAIN, SG_INFO, "Loading tile " << basename.str() );
+        SG_LOG( SG_TERRAIN, SG_INFO, "  Trying " << basename.str() );
 
-#define FG_MAX_LIGHTS 1000
 
         // Check for master .stg (scene terra gear) file
         SGPath stg_name = basename;
         stg_name.concat( ".stg" );
 
         sg_gzifstream in( stg_name.str() );
+        if ( !in.is_open() )
+            continue;
 
-        if ( in.is_open() ) {
-            string token, name;
+        while ( ! in.eof() ) {
+            string token;
+            in >> token;
 
-            while ( ! in.eof() ) {
-                in >> token;
+            if ( token[0] == '#' ) {
+               in >> ::skipeol;
+               continue;
+            }
+                            // Load only once (first found)
+            if ( token == "OBJECT_BASE" ) {
+                string name;
+                in >> name >> ::skipws;
+                SG_LOG( SG_TERRAIN, SG_INFO, "    " << token << " " << name );
 
-                if ( token[0] == '#' ) {
-                   in >> ::skipeol;
-                   continue;
+                if (!found_tile_base) {
+                    found_tile_base = true;
+                    has_base = true;
+
+                    object_base = tile_path;
+                    object_base.append(name);
+
+                } else
+                    SG_LOG(SG_TERRAIN, SG_INFO, "    (skipped)");
+
+                            // Load only if base is not in another file
+            } else if ( token == "OBJECT" ) {
+                if (!found_tile_base || has_base)
+                    objects.push_back(new Object(OBJECT, token, tile_path, in));
+                else {
+                    string name;
+                    in >> name >> ::skipeol;
+                    SG_LOG(SG_TERRAIN, SG_INFO, "    " << token << "  "
+                            << name << "  (skipped)");
                 }
-                                // Load only once (first found)
-                if ( token == "OBJECT_BASE" ) {
-                    in >> name >> ::skipws;
-                    SG_LOG( SG_TERRAIN, SG_INFO, "token = " << token
-                            << " name = " << name );
 
-                    if (!found_tile_base) {
-                        found_tile_base = true;
-                        has_base = true;
+                            // Always OK to load
+            } else if ( token == "OBJECT_STATIC" ) {
+                objects.push_back(new Object(OBJECT_STATIC, token, tile_path, in));
 
-                        SGPath custom_path = tile_path;
-                        custom_path.append( name );
+            } else if ( token == "OBJECT_SHARED" ) {
+                objects.push_back(new Object(OBJECT_SHARED, token, tile_path, in));
 
-                        ssgBranch *geometry = new ssgBranch;
-                        if ( obj_load( custom_path.str(),
-                                       geometry, NULL, NULL, NULL, light_pts,
-                                       true ) )
-                            {
-                                geometry->getKid( 0 )->setTravCallback( 
-                                                        SSG_CALLBACK_PRETRAV,
-                                                        &FGTileMgr::tile_filter_cb );
-                                new_tile -> addKid( geometry );
-                            } else {
-                                delete geometry;
-                            }
-                    } else {
-                        SG_LOG( SG_TERRAIN, SG_INFO, " (skipped)" );
-                    }
+            } else if ( token == "OBJECT_TAXI_SIGN" ) {
+                objects.push_back(new Object(OBJECT_TAXI_SIGN, token, tile_path, in));
 
-                                // Load only if base is not in another file
-                } else if ( token == "OBJECT" ) {
-                    if (!found_tile_base || has_base) {
-                        in >> name >> ::skipws;
-                        SG_LOG( SG_TERRAIN, SG_INFO, "token = " << token
-                                << " name = " << name );
+            } else if ( token == "OBJECT_RUNWAY_SIGN" ) {
+                objects.push_back(new Object(OBJECT_RUNWAY_SIGN, token, tile_path, in));
 
-                        SGPath custom_path = tile_path;
-                        custom_path.append( name );
-
-                        ssgBranch *geometry = new ssgBranch;
-                        ssgBranch *vasi_lights = new ssgBranch;
-                        ssgBranch *rwy_lights = new ssgBranch;
-                        ssgBranch *taxi_lights = new ssgBranch;
-                        if ( obj_load( custom_path.str(),
-                                       geometry, vasi_lights, rwy_lights,
-                                       taxi_lights, NULL, false ) )
-                            {
-                                if ( geometry -> getNumKids() > 0 ) {
-                                    geometry->getKid( 0 )->setTravCallback(
-                                                                SSG_CALLBACK_PRETRAV,
-                                                                &FGTileMgr::tile_filter_cb );
-                                    new_tile -> addKid( geometry );
-                                } else {
-                                    delete geometry;
-                                }
-                                if ( vasi_lights -> getNumKids() > 0 ) {
-                                    vasi_lights_transform -> addKid( vasi_lights );
-                                } else {
-                                    delete vasi_lights;
-                                }
-                                if ( rwy_lights -> getNumKids() > 0 ) {
-                                    rwy_lights_transform -> addKid( rwy_lights );
-                                } else {
-                                    delete rwy_lights;
-                                }
-                                if ( taxi_lights -> getNumKids() > 0 ) {
-                                    taxi_lights_transform -> addKid( taxi_lights );
-                                } else {
-                                    delete taxi_lights;
-                                }
-                            } else {
-                                delete geometry;
-                                delete vasi_lights;
-                                delete rwy_lights;
-                                delete taxi_lights;
-                            }
-                    }
-
-                                // Always OK to load
-                } else if ( token == "OBJECT_STATIC" ||
-                            token == "OBJECT_SHARED" ) {
-                    // load object info
-                    double lon, lat, elev, hdg;
-                    in >> name >> lon >> lat >> elev >> hdg >> ::skipws;
-                    SG_LOG( SG_TERRAIN, SG_INFO, "token = " << token
-                            << " name = " << name 
-                            << " pos = " << lon << ", " << lat
-                            << " elevation = " << elev
-                            << " heading = " << hdg );
-
-                    // object loading is deferred to main render thread,
-                    // but lets figure out the paths right now.
-                    SGPath custom_path;
-                    if ( token == "OBJECT_STATIC" ) {
-                        custom_path= tile_path;
-                    } else {
-                        custom_path = globals->get_fg_root();
-                    }
-                    custom_path.append( name );
-
-                    sgCoord obj_pos;
-                    WorldCoordinate( &obj_pos, center, lat, lon, elev, hdg );
-                
-                    ssgTransform *obj_trans = new ssgTransform;
-                    obj_trans->setTransform( &obj_pos );
-
-                    // wire as much of the scene graph together as we can
-                    new_tile->addKid( obj_trans );
-
-                    // bump up the pending models count
-                    pending_models++;
-
-                    // push an entry onto the model load queue
-                    FGDeferredModel *dm
-                        = new FGDeferredModel( custom_path.str(),
-                                               tile_path.str(),
-                                               tile_bucket,
-                                               this, obj_trans );
-                    FGTileMgr::model_ready( dm );
-
-                                // Do we even use this one?
-                } else if ( token == "OBJECT_TAXI_SIGN" ) {
-                    // load object info
-                    double lon, lat, elev, hdg;
-                    in >> name >> lon >> lat >> elev >> hdg >> ::skipws;
-                    SG_LOG( SG_TERRAIN, SG_INFO, "token = " << token
-                            << " name = " << name 
-                            << " pos = " << lon << ", " << lat
-                            << " elevation = " << elev
-                            << " heading = " << hdg );
-
-                    // load the object itself
-                    SGPath custom_path = tile_path;
-                    custom_path.append( name );
-
-                    sgCoord obj_pos;
-                    WorldCoordinate( &obj_pos, center, lat, lon, elev, hdg );
-
-                    ssgTransform *obj_trans = new ssgTransform;
-                    obj_trans->setTransform( &obj_pos );
-
-                    ssgBranch *custom_obj
-                        = sgMakeTaxiSign( globals->get_matlib(),
-                                          custom_path.str(), name );
-
-                    // wire the pieces together
-                    if ( custom_obj != NULL ) {
-                        obj_trans -> addKid( custom_obj );
-                    }
-                    new_tile->addKid( obj_trans );
-
-                                // Do we even use this one?
-                } else if ( token == "OBJECT_RUNWAY_SIGN" ) {
-                    // load object info
-                    double lon, lat, elev, hdg;
-                    in >> name >> lon >> lat >> elev >> hdg >> ::skipws;
-                    SG_LOG( SG_TERRAIN, SG_INFO, "token = " << token
-                            << " name = " << name 
-                            << " pos = " << lon << ", " << lat
-                            << " elevation = " << elev
-                            << " heading = " << hdg );
-
-                    // load the object itself
-                    SGPath custom_path = tile_path;
-                    custom_path.append( name );
-
-                    sgCoord obj_pos;
-                    WorldCoordinate( &obj_pos, center, lat, lon, elev, hdg );
-
-                    ssgTransform *obj_trans = new ssgTransform;
-                    obj_trans->setTransform( &obj_pos );
-
-                    ssgBranch *custom_obj
-                        = sgMakeRunwaySign( globals->get_matlib(),
-                                            custom_path.str(), name );
-
-                    // wire the pieces together
-                    if ( custom_obj != NULL ) {
-                        obj_trans -> addKid( custom_obj );
-                    }
-                    new_tile->addKid( obj_trans );
-
-                                // I don't think we use this, either
-                } else if ( token == "RWY_LIGHTS" ) {
-                    double lon, lat, hdg, len, width;
-                    string common, end1, end2;
-                    in >> lon >> lat >> hdg >> len >> width
-                       >> common >> end1 >> end2;
-                    SG_LOG( SG_TERRAIN, SG_INFO, "token = " << token
-                            << " pos = " << lon << ", " << lat
-                            << " hdg = " << hdg
-                            << " size = " << len << ", " << width
-                            << " codes = " << common << " "
-                            << end1 << " " << end2 );
-                } else {
-                    SG_LOG( SG_TERRAIN, SG_DEBUG,
-                            "Unknown token " << token << " in "
-                            << stg_name.str() );
-                    in >> ::skipws;
-                }
+            } else {
+                SG_LOG( SG_TERRAIN, SG_DEBUG,
+                        "Unknown token '" << token << "' in " << stg_name.str() );
+                in >> ::skipws;
             }
         }
-
-        i++;
     }
 
-    if ( !found_tile_base ) {
-        // no tile base found, generate an ocean tile on the fly for
-        // this area
-        ssgBranch *geometry = new ssgBranch;
+
+    // obj_load() will generate ground lighting for us ...
+    ssgVertexArray *light_pts = new ssgVertexArray( 100 );
+    ssgBranch* new_tile = new ssgBranch;
+
+
+    if (found_tile_base) {
+        // load tile if found ...
+        ssgSharedPtr<ssgBranch> geometry = new ssgBranch;
+        if ( obj_load( object_base.str(), geometry,
+                       NULL, NULL, NULL, light_pts, true ) ) {
+            geometry->getKid( 0 )->setTravCallback(SSG_CALLBACK_PRETRAV,
+                    &FGTileMgr::tile_filter_cb);
+
+            new_tile -> addKid( geometry );
+        }
+
+    } else {
+        // ... or generate an ocean tile on the fly
+        SG_LOG(SG_TERRAIN, SG_INFO, "  Generating ocean tile");
+        ssgSharedPtr<ssgBranch> geometry = new ssgBranch;
         Point3D c;
         double br;
         if ( sgGenTile( path_list[0], tile_bucket, &c, &br,
-                        globals->get_matlib(), geometry ) )
-        {
+                        globals->get_matlib(), geometry ) ) {
             center = c;
             bounding_radius = br;
             new_tile -> addKid( geometry );
         } else {
-            delete geometry;
             SG_LOG( SG_TERRAIN, SG_ALERT,
                     "Warning: failed to generate ocean tile!" );
         }
     }
 
+
+    // now that we have a valid center, process all the objects
+    for (unsigned int j = 0; j < objects.size(); j++) {
+        const Object *obj = objects[j];
+
+        if (obj->type == OBJECT) {
+            SGPath custom_path = obj->path;
+            custom_path.append( obj->name );
+
+            ssgSharedPtr<ssgBranch> geometry = new ssgBranch;
+            ssgSharedPtr<ssgBranch> vasi_lights = new ssgBranch;
+            ssgSharedPtr<ssgBranch> rwy_lights = new ssgBranch;
+            ssgSharedPtr<ssgBranch> taxi_lights = new ssgBranch;
+
+            if ( obj_load( custom_path.str(),
+                           geometry, vasi_lights, rwy_lights,
+                           taxi_lights, NULL, false ) ) {
+
+                if ( geometry -> getNumKids() > 0 ) {
+                    geometry->getKid( 0 )->setTravCallback(
+                                                SSG_CALLBACK_PRETRAV,
+                                                &FGTileMgr::tile_filter_cb );
+                    new_tile -> addKid( geometry );
+                }
+
+                if ( vasi_lights -> getNumKids() > 0 )
+                    vasi_lights_transform -> addKid( vasi_lights );
+
+                if ( rwy_lights -> getNumKids() > 0 )
+                    rwy_lights_transform -> addKid( rwy_lights );
+
+                if ( taxi_lights -> getNumKids() > 0 )
+                    taxi_lights_transform -> addKid( taxi_lights );
+            }
+
+
+        } else if (obj->type == OBJECT_SHARED || obj->type == OBJECT_STATIC) {
+            // object loading is deferred to main render thread,
+            // but lets figure out the paths right now.
+            SGPath custom_path;
+            if ( obj->type == OBJECT_STATIC ) {
+                custom_path = obj->path;
+            } else {
+                custom_path = globals->get_fg_root();
+            }
+            custom_path.append( obj->name );
+
+            sgCoord obj_pos;
+            WorldCoordinate( &obj_pos, center, obj->lat, obj->lon, obj->elev, obj->hdg );
+
+            ssgTransform *obj_trans = new ssgTransform;
+            obj_trans->setTransform( &obj_pos );
+
+            // wire as much of the scene graph together as we can
+            new_tile->addKid( obj_trans );
+            pending_models++;
+
+            // push an entry onto the model load queue
+            FGDeferredModel *dm
+                = new FGDeferredModel( custom_path.str(),
+                                       obj->path.str(),
+                                       tile_bucket,
+                                       this, obj_trans,
+                                       obj->type == OBJECT_SHARED );
+            FGTileMgr::model_ready( dm );
+
+
+        } else if (obj->type == OBJECT_TAXI_SIGN || obj->type == OBJECT_RUNWAY_SIGN) {
+            ssgBranch *(*make_sign)(SGMaterialLib *, const string, const string);
+            make_sign = obj->type == OBJECT_TAXI_SIGN ? sgMakeTaxiSign : sgMakeRunwaySign;
+
+            // load the object itself
+            SGPath custom_path = obj->path;
+            custom_path.append( obj->name );
+
+            sgCoord obj_pos;
+            WorldCoordinate( &obj_pos, center, obj->lat, obj->lon, obj->elev, obj->hdg );
+
+            ssgTransform *obj_trans = new ssgTransform;
+            obj_trans->setTransform( &obj_pos );
+
+            ssgBranch *custom_obj
+                = (*make_sign)(globals->get_matlib(), custom_path.str(), obj->name);
+
+            // wire the pieces together
+            if ( custom_obj != NULL ) {
+                obj_trans -> addKid( custom_obj );
+            }
+            new_tile->addKid( obj_trans );
+
+        }
+        delete obj;
+    }
+
+
     if ( new_tile != NULL ) {
         terra_range->addKid( new_tile );
     }
@@ -1011,7 +991,6 @@ FGTileEntry::add_ssg_nodes( ssgBranch *terrain_branch,
     }
 #endif
 
-    terra_transform->ref();
     terrain_branch->addKid( terra_transform );
     globals->get_scenery()->register_placement_transform(terra_transform);
 
@@ -1024,7 +1003,6 @@ FGTileEntry::add_ssg_nodes( ssgBranch *terrain_branch,
     if ( gnd_lights_transform != NULL ) {
         // bump up the ref count so we can remove this later without
         // having ssg try to free the memory.
-        gnd_lights_transform->ref();
         gnd_lights_branch->addKid( gnd_lights_transform );
         globals->get_scenery()->register_placement_transform(gnd_lights_transform);
     }
@@ -1032,7 +1010,6 @@ FGTileEntry::add_ssg_nodes( ssgBranch *terrain_branch,
     if ( vasi_lights_transform != NULL ) {
         // bump up the ref count so we can remove this later without
         // having ssg try to free the memory.
-        vasi_lights_selector->ref();
         vasi_lights_selector->addKid( vasi_lights_transform );
         globals->get_scenery()->register_placement_transform(vasi_lights_transform);
         vasi_lights_branch->addKid( vasi_lights_selector );
@@ -1041,7 +1018,6 @@ FGTileEntry::add_ssg_nodes( ssgBranch *terrain_branch,
     if ( rwy_lights_transform != NULL ) {
         // bump up the ref count so we can remove this later without
         // having ssg try to free the memory.
-        rwy_lights_selector->ref();
         rwy_lights_selector->addKid( rwy_lights_transform );
         globals->get_scenery()->register_placement_transform(rwy_lights_transform);
         rwy_lights_branch->addKid( rwy_lights_selector );
@@ -1050,7 +1026,6 @@ FGTileEntry::add_ssg_nodes( ssgBranch *terrain_branch,
     if ( taxi_lights_transform != NULL ) {
         // bump up the ref count so we can remove this later without
         // having ssg try to free the memory.
-        taxi_lights_selector->ref();
         taxi_lights_selector->addKid( taxi_lights_transform );
         globals->get_scenery()->register_placement_transform(taxi_lights_transform);
         taxi_lights_branch->addKid( taxi_lights_selector );
