@@ -1,3 +1,8 @@
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <string.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -74,7 +79,7 @@ bool FGNasalSys::parseAndRun(const char* sourceCode)
     naCall(_context, code, 0, 0, naNil(), naNil());
 
     if(!naGetError(_context)) return true;
-    logError();
+    logError(_context);
     return false;
 }
 
@@ -234,6 +239,22 @@ static naRef f_settimer(naContext c, naRef me, int argc, naRef* args)
     return naNil();
 }
 
+// setlistener(func, property, bool) extension function.  Falls through to
+// FGNasalSys::setListener().  See there for docs.
+static naRef f_setlistener(naContext c, naRef me, int argc, naRef* args)
+{
+    FGNasalSys* nasal = (FGNasalSys*)globals->get_subsystem("nasal");
+    return nasal->setListener(argc, args);
+}
+
+// removelistener(int) extension function. Falls through to
+// FGNasalSys::removeListener(). See there for docs.
+static naRef f_removelistener(naContext c, naRef me, int argc, naRef* args)
+{
+    FGNasalSys* nasal = (FGNasalSys*)globals->get_subsystem("nasal");
+    return nasal->removeListener(argc, args);
+}
+
 // Returns a ghost handle to the argument to the currently executing
 // command
 static naRef f_cmdarg(naContext c, naRef me, int argc, naRef* args)
@@ -278,6 +299,29 @@ static naRef f_rand(naContext c, naRef me, int argc, naRef* args)
     return naNum(sg_random());
 }
 
+static naRef f_srand(naContext c, naRef me, int argc, naRef* args)
+{
+    sg_srandom_time();
+    return naNum(0);
+}
+
+// Return an array listing of all files in a directory
+static naRef f_directory(naContext c, naRef me, int argc, naRef* args)
+{
+    if(argc != 1 || !naIsString(args[0]))
+        naRuntimeError(c, "bad arguments to directory()");
+    naRef ldir = args[0];
+    ulDir* dir = ulOpenDir(naStr_data(args[0]));
+    if(!dir) return naNil();
+    naRef result = naNewVector(c);
+    ulDirEnt* dent;
+    while((dent = ulReadDir(dir)))
+        naVec_append(result, naStr_fromdata(naNewString(c), dent->d_name,
+                                            strlen(dent->d_name)));
+    ulCloseDir(dir);
+    return result;
+}
+
 // Table of extension functions.  Terminate with zeros.
 static struct { char* name; naCFunction func; } funcs[] = {
     { "getprop",   f_getprop },
@@ -285,9 +329,13 @@ static struct { char* name; naCFunction func; } funcs[] = {
     { "print",     f_print },
     { "_fgcommand", f_fgcommand },
     { "settimer",  f_settimer },
+    { "_setlistener", f_setlistener },
+    { "removelistener", f_removelistener },
     { "_cmdarg",  f_cmdarg },
     { "_interpolate",  f_interpolate },
     { "rand",  f_rand },
+    { "srand",  f_srand },
+    { "directory", f_directory },
     { 0, 0 }
 };
 
@@ -313,6 +361,11 @@ void FGNasalSys::init()
     // Add in the math library under "math"
     hashset(_globals, "math", naMathLib(_context));
 
+    // Add in the IO library.  Disabled currently until after the
+    // 0.9.10 release.
+    // hashset(_globals, "io", naIOLib(_context));
+    // hashset(_globals, "bits", naBitsLib(_context));
+
     // Add our custom extension functions:
     for(i=0; funcs[i].name; i++)
         hashset(_globals, funcs[i].name,
@@ -339,6 +392,7 @@ void FGNasalSys::init()
         if(file.extension() != "nas") continue;
         loadModule(fullpath, file.base().c_str());
     }
+    ulCloseDir(dir);
 
     // Pull scripts out of the property tree, too
     loadPropertyScripts();
@@ -395,17 +449,17 @@ void FGNasalSys::loadPropertyScripts()
 }
 
 // Logs a runtime error, with stack trace, to the FlightGear log stream
-void FGNasalSys::logError()
+void FGNasalSys::logError(naContext context)
 {
     SG_LOG(SG_NASAL, SG_ALERT,
-           "Nasal runtime error: " << naGetError(_context));
+           "Nasal runtime error: " << naGetError(context));
     SG_LOG(SG_NASAL, SG_ALERT,
-           "  at " << naStr_data(naGetSourceFile(_context, 0)) <<
-           ", line " << naGetLine(_context, 0));
-    for(int i=1; i<naStackDepth(_context); i++)
+           "  at " << naStr_data(naGetSourceFile(context, 0)) <<
+           ", line " << naGetLine(context, 0));
+    for(int i=1; i<naStackDepth(context); i++)
         SG_LOG(SG_NASAL, SG_ALERT,
-               "  called from: " << naStr_data(naGetSourceFile(_context, i)) <<
-               ", line " << naGetLine(_context, i));
+               "  called from: " << naStr_data(naGetSourceFile(context, i)) <<
+               ", line " << naGetLine(context, i));
 }
 
 // Reads a script file, executes it, and places the resulting
@@ -446,12 +500,21 @@ void FGNasalSys::createModule(const char* moduleName, const char* fileName,
 
     naCall(_context, code, 0, 0, naNil(), locals);
     if(naGetError(_context)) {
-        logError();
+        logError(_context);
         return;
     }
     hashset(_globals, moduleName, locals);
 }
 
+void FGNasalSys::deleteModule(const char* moduleName)
+{
+    naRef modname = naNewString(_context);
+    naStr_fromdata(modname, (char*)moduleName, strlen(moduleName));
+    naModLock();
+    naHash_delete(_globals, modname);
+    naModUnlock();
+}
+
 naRef FGNasalSys::parse(const char* filename, const char* buf, int len)
 {
     int errLine = -1;
@@ -476,12 +539,13 @@ bool FGNasalSys::handleCommand(const SGPropertyNode* arg)
     naRef code = parse(arg->getPath(true), nasal, strlen(nasal));
     if(naIsNil(code)) return false;
 
+    naContext c = naNewContext();
     naRef locals = naNil();
-    if (moduleName[0]) {
-        naRef modname = naNewString(_context);
+    if(moduleName[0]) {
+        naRef modname = naNewString(c);
         naStr_fromdata(modname, (char*)moduleName, strlen(moduleName));
         if(!naHash_get(_globals, modname, &locals))
-            locals = naNewHash(_context);
+            locals = naNewHash(c);
     }
     // Cache the command argument for inspection via cmdarg().  For
     // performance reasons, we won't bother with it if the invoked
@@ -489,10 +553,15 @@ bool FGNasalSys::handleCommand(const SGPropertyNode* arg)
     _cmdArg = (SGPropertyNode*)arg;
 
     // Call it!
-    naRef result = naCall(_context, code, 0, 0, naNil(), locals);
-    if(!naGetError(_context)) return true;
-    logError();
-    return false;
+    naModUnlock();
+    naRef result = naCall(c, code, 0, 0, naNil(), locals);
+    naModLock();
+    bool error = naGetError(c);
+    if(error)
+       logError(c);
+
+    naFreeContext(c);
+    return !error;
 }
 
 // settimer(func, dt, simtime) extension function.  The first argument
@@ -514,7 +583,7 @@ void FGNasalSys::setTimer(int argc, naRef* args)
 
     naRef delta = argc > 1 ? args[1] : naNil();
     if(naIsNil(delta)) return;
-    
+
     bool simtime = (argc > 2 && naTrue(args[2])) ? false : true;
 
     // Generate and register a C++ timer handler
@@ -532,7 +601,7 @@ void FGNasalSys::handleTimer(NasalTimer* t)
 {
     naCall(_context, t->handler, 0, 0, naNil(), naNil());
     if(naGetError(_context))
-        logError();
+        logError(_context);
     gcRelease(t->gcKey);
 }
 
@@ -553,3 +622,137 @@ void FGNasalSys::NasalTimer::timerExpired()
     nasal->handleTimer(this);
     delete this;
 }
+
+int FGNasalSys::_listenerId = 0;
+
+// setlistener(property, func, bool) extension function.  The first argument
+// is either a ghost (SGPropertyNode_ptr*) or a string (global property
+// path), the second is a Nasal function, the optional third one a bool.
+// If the bool is true, then the listener is executed initially. The
+// setlistener() function returns a unique id number, that can be used
+// as argument to the removelistener() function.
+naRef FGNasalSys::setListener(int argc, naRef* args)
+{
+    SGPropertyNode_ptr node;
+    naRef prop = argc > 0 ? args[0] : naNil();
+    if(naIsString(prop)) node = fgGetNode(naStr_data(prop), true);
+    else if(naIsGhost(prop)) node = *(SGPropertyNode_ptr*)naGhost_ptr(prop);
+    else return naNil();
+
+    naRef handler = argc > 1 ? args[1] : naNil();
+    if(!(naIsCode(handler) || naIsCCode(handler) || naIsFunc(handler)))
+        return naNil();
+
+    bool initial = argc > 2 && naTrue(args[2]);
+
+    FGNasalListener *nl = new FGNasalListener(node, handler, this,
+            gcSave(handler));
+    node->addChangeListener(nl, initial);
+
+    _listener[_listenerId] = nl;
+    return naNum(_listenerId++);
+}
+
+// removelistener(int) extension function. The argument is the id of
+// a listener as returned by the setlistener() function.
+naRef FGNasalSys::removeListener(int argc, naRef* args)
+{
+    naRef id = argc > 0 ? args[0] : naNil();
+    if(!naIsNum(id))
+        return naNil();
+
+    int i = int(id.num);
+    if (_listener.find(i) == _listener.end())
+        return naNil();
+
+    FGNasalListener *nl = _listener[i];
+    nl->_node->removeChangeListener(nl);
+    _listener.erase(i);
+    delete nl;
+    return naNum(_listener.size());
+}
+
+
+
+// FGNasalListener class.
+
+FGNasalListener::FGNasalListener(SGPropertyNode_ptr node, naRef handler,
+                                 FGNasalSys* nasal, int key) :
+    _node(node),
+    _handler(handler),
+    _gcKey(key),
+    _nas(nasal),
+    _active(0)
+{
+}
+
+FGNasalListener::~FGNasalListener()
+{
+    _nas->gcRelease(_gcKey);
+}
+
+void FGNasalListener::valueChanged(SGPropertyNode* node)
+{
+    if (_active) {
+        SG_LOG(SG_NASAL, SG_ALERT, "Recursive listener call "
+                "on property " << node->getPath());
+        return;
+    }
+    _active++;
+    _nas->_cmdArg = node;
+    naContext c = naNewContext();
+    naModUnlock();
+    naCall(c, _handler, 0, 0, naNil(), naNil());
+    naModLock();
+    if(naGetError(c))
+        _nas->logError(c);
+    naFreeContext(c);
+    _active--;
+}
+
+
+
+
+// FGNasalModelData class.  If sgLoad3DModel() is called with a pointer to
+// such a class, then it lets modelLoaded() run the <load> script, and the
+// destructor the <unload> script. The latter happens when the model branch
+// is removed from the scene graph.
+
+void FGNasalModelData::modelLoaded(const string& path, SGPropertyNode *prop,
+                                   ssgBranch *)
+{
+    SGPropertyNode *n = prop->getNode("nasal"), *load;
+    if (!n)
+        return;
+
+    load = n->getNode("load");
+    _unload = n->getNode("unload");
+    if (!load && !_unload)
+        return;
+
+    _module = path;
+    const char *s = load ? load->getStringValue() : "";
+    FGNasalSys *nas = (FGNasalSys *)globals->get_subsystem("nasal");
+    nas->createModule(_module.c_str(), _module.c_str(), s, strlen(s));
+}
+
+FGNasalModelData::~FGNasalModelData()
+{
+    if (_module.empty())
+        return;
+
+    FGNasalSys *nas = (FGNasalSys *)globals->get_subsystem("nasal");
+    if (!nas) {
+        SG_LOG(SG_NASAL, SG_ALERT, "Trying to run an <unload> script "
+                "without Nasal subsystem present.");
+        return;
+    }
+
+    if (_unload) {
+        const char *s = _unload->getStringValue();
+        nas->createModule(_module.c_str(), _module.c_str(), s, strlen(s));
+    }
+    nas->deleteModule(_module.c_str());
+}
+
+
