@@ -17,9 +17,9 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: light.cxx,v 1.50 2005/11/11 14:46:15 ehofman Exp $
+// $Id: light.cxx,v 1.53 2006-02-21 01:20:18 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -132,11 +132,29 @@ void FGLight::reinit () {
 void FGLight::bind () {
     SGPropertyNode *prop = globals->get_props();
     prop->tie("/sim/time/sun-angle-rad",SGRawValuePointer<double>(&_sun_angle));
+    prop->tie("/rendering/scene/ambient/red",SGRawValuePointer<float>(&_scene_ambient[0]));
+    prop->tie("/rendering/scene/ambient/green",SGRawValuePointer<float>(&_scene_ambient[1]));
+    prop->tie("/rendering/scene/ambient/blue",SGRawValuePointer<float>(&_scene_ambient[2]));
+    prop->tie("/rendering/scene/diffuse/red",SGRawValuePointer<float>(&_scene_diffuse[0]));
+    prop->tie("/rendering/scene/diffuse/green",SGRawValuePointer<float>(&_scene_diffuse[1]));
+    prop->tie("/rendering/scene/diffuse/blue",SGRawValuePointer<float>(&_scene_diffuse[2]));
+    prop->tie("/rendering/scene/specular/red",SGRawValuePointer<float>(&_scene_specular[0]));
+    prop->tie("/rendering/scene/specular/green",SGRawValuePointer<float>(&_scene_specular[1]));
+    prop->tie("/rendering/scene/specular/blue",SGRawValuePointer<float>(&_scene_specular[2]));
 }
 
 void FGLight::unbind () {
     SGPropertyNode *prop = globals->get_props();
     prop->untie("/sim/time/sun-angle-rad");
+    prop->untie("/rendering/scene/ambient/red");
+    prop->untie("/rendering/scene/ambient/green");
+    prop->untie("/rendering/scene/ambient/blue");
+    prop->untie("/rendering/scene/diffuse/red");
+    prop->untie("/rendering/scene/diffuse/green");
+    prop->untie("/rendering/scene/diffuse/blue");
+    prop->untie("/rendering/scene/specular/red");
+    prop->untie("/rendering/scene/specular/green");
+    prop->untie("/rendering/scene/specular/blue");
 }
 
 
