@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  *
  **************************************************************************/
@@ -33,6 +33,11 @@
  *
  *
  *****************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <time.h>
 #include <iostream>
@@ -85,7 +90,6 @@ void FGTrafficManager::init()
 	{
 	  currAircraft=scheduledAircraft.erase(currAircraft);
 	  //cerr << "Erasing " << currAircraft->getRegistration() << endl;
-	  currAircraft--;
 	}
       else 
 	{
@@ -102,6 +106,8 @@ void FGTrafficManager::init()
 void FGTrafficManager::update(double something)
 {
   time_t now = time(NULL) + fgGetLong("/sim/time/warp");
+  if (scheduledAircraft.size() == 0)
+	  return;
   if(currAircraft == scheduledAircraft.end())
     {
       //cerr << "resetting schedule " << endl;
@@ -262,11 +268,9 @@ void  FGTrafficManager::pi (const char * target, const char * data) {
 }
 
 void  FGTrafficManager::warning (const char * message, int line, int column) {
-  cout << "Warning: " << message << " (" << line << ',' << column << ')'   
-       << endl;
+  SG_LOG(SG_IO, SG_WARN, "Warning: " << message << " (" << line << ',' << column << ')');
 }
 
 void  FGTrafficManager::error (const char * message, int line, int column) {
-  cout << "Error: " << message << " (" << line << ',' << column << ')'
-       << endl;
+  SG_LOG(SG_IO, SG_ALERT, "Error: " << message << " (" << line << ',' << column << ')');
 }
